/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "coverselectormodel.h"

CoverSelectorModel::CoverSelectorModel(CoverSelectorModel *master, QObject *parent) :
    QAbstractItemModel(parent),
    m_Master(master)
{
}

bool CoverSelectorModel::insertRows (int row, int count, const QModelIndex& parent)
{
    beginInsertRows(parent, row, row+count);
    for (int i=0;i<count;i++) m_Covers.insert(row,0l);
    endInsertRows();
    return true;
}

bool CoverSelectorModel::removeRows(int row, int count, const QModelIndex &parent)
{
    beginRemoveRows(parent,row,row+count);
    for (int i=0;i<count;i++) {
        m_Covers[row]->deleteLater();
        m_Covers.removeAt(row);
    }
    endRemoveRows();
    return true;
}

QModelIndex CoverSelectorModel::index (int row, int column, const QModelIndex&) const
{
    return createIndex(row,column);
}

QModelIndex CoverSelectorModel::parent (const QModelIndex&) const
{
    return QModelIndex();
}

int CoverSelectorModel::rowCount (const QModelIndex&) const
{
    return m_Covers.size();
}

Qt::ItemFlags CoverSelectorModel::flags (const QModelIndex&) const
{
    return Qt::ItemIsEnabled;
}

QVariant CoverSelectorModel::data (const QModelIndex& index, int role) const
{
    if (!index.isValid() && index.column()!=0) {
        return QVariant();
    }

    switch (role) {
    case Qt::DisplayRole:
    {
        const Texel::CoverSearch::Cover* cover=getCover(index);
        if (cover!=0l) {
            QString html;
            if (cover->hasSize()) {
                html+=QString("<h3>Size: ")+QString::number(cover->getSize().width())+" x "+QString::number(cover->getSize().height())+"</h3>";
            }
            if (cover->hasBytes()) {
                html+=QString("<p>Filesize: "+QString::number(cover->getBytes()/1024)+" kB</p>");
            }
            if (!cover->description().isEmpty()) {
                html+="<p>"+cover->description()+"</p>";
            }
            if (!cover->getSourceDomain().isEmpty()) {
                html+=QString("<p>Origin: ")+cover->getSourceDomain()+"</p>";
            }
            return html;
        }
        break;
    }
    case Qt::DecorationRole:
    {
        const Texel::CoverSearch::Cover* cover=getCover(index);
        if (cover!=0l) {
            return cover->getPreview();
        }
        break;
    }
    default:
        ;
    }

    return QVariant();
}

bool CoverSelectorModel::setData (const QModelIndex& index, const QVariant& value, int role)
{
    if (index.isValid() && index.column()==0) {
        switch (role) {
        case Qt::EditRole:
            Texel::CoverSearch::Cover* cover=qobject_cast<Texel::CoverSearch::Cover*>(value.value<QObject*>());
            if (cover!=0l) {
                m_Covers[index.row()]=cover;
                cover->setAlbum(album());
                cover->setArtist(artist());
                cover->setReference(getReference());
                connect (cover, SIGNAL(imageReceived()), this, SLOT(updatedCover()));
                return true;
            }
        }
    }

    return false;
}

int CoverSelectorModel::columnCount (const QModelIndex&) const
{
    return 1;
}

void CoverSelectorModel::updatedCover ()
{
    Texel::CoverSearch::Cover* cover=dynamic_cast<Texel::CoverSearch::Cover*>(sender());
    const int pos=m_Covers.indexOf(cover);
    if (pos>=0) {
        emit dataChanged(index(pos,0),index(pos,0));
    }
}

const Texel::CoverSearch::Cover *CoverSelectorModel::getCover (const QModelIndex& index) const
{
    if (index.isValid() && index.column()==0 && index.row()>=0 && index.row()<m_Covers.size()) {
        return m_Covers[index.row()];
    }
    return 0l;
}

Texel::CoverSearch::Cover *CoverSelectorModel::getCover (const QModelIndex& index)
{
    if (index.isValid() && index.column()==0 && index.row()>=0 && index.row()<m_Covers.size()) {
        return m_Covers[index.row()];
    }
    return 0l;
}
