/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef COVERSELECTORMODEL_H
#define COVERSELECTORMODEL_H

#include <QAbstractItemModel>
#include <QList>

#include "cover.h"

class CoverSelectorModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    explicit CoverSelectorModel(CoverSelectorModel* master=0, QObject *parent = 0);
    
    virtual bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());
    virtual bool insertRows (int row, int count, const QModelIndex& parent = QModelIndex());
    virtual QModelIndex index (int row, int column, const QModelIndex& parent = QModelIndex()) const;
    virtual QModelIndex parent (const QModelIndex& index) const;
    virtual int rowCount (const QModelIndex& parent = QModelIndex()) const;
    virtual Qt::ItemFlags flags (const QModelIndex& index) const;
    virtual QVariant data (const QModelIndex& index, int role = Qt::DisplayRole) const;
    virtual int columnCount (const QModelIndex& parent = QModelIndex()) const;

    virtual bool setData (const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);

    Texel::CoverSearch::Cover* getCover(const QModelIndex& index);
    const Texel::CoverSearch::Cover* getCover(const QModelIndex& index) const;

    CoverSelectorModel* master () const;

    void setArtistAndAlbum (const QString& artist, const QString& album);
    QString album () const;
    QString artist () const;

    void setReference ( const QVariant& ref);
    QVariant getReference () const;

signals:
    
private slots:
    void updatedCover ();

private:
    QString m_Artist;
    QString m_Album;

    // Reference to which item this model belongs to
    QVariant m_Reference;

    QList<Texel::CoverSearch::Cover*> m_Covers;
    CoverSelectorModel* m_Master;
};

inline CoverSelectorModel* CoverSelectorModel::master () const
{
    return m_Master;
}

inline void CoverSelectorModel::setArtistAndAlbum (const QString& artist, const QString& album)
{
    m_Artist=artist;
    m_Album=album;
}

inline QString CoverSelectorModel::album () const
{
    return m_Album;
}

inline QString CoverSelectorModel::artist () const
{
    return m_Artist;
}

inline void CoverSelectorModel::setReference ( const QVariant& ref)
{
    m_Reference=ref;
}

inline QVariant CoverSelectorModel::getReference () const
{
    return m_Reference;
}


#endif // COVERSELECTORMODEL_H
