/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*************************************************************************/

#include "draganddroplabel.h"

#include "aid.h"

#include <QDragEnterEvent>
#include <QString>
#include <QToolTip>

using namespace std;

namespace Texel {
    namespace CoverSearch {

        DragAndDropLabel::DragAndDropLabel(QWidget *parent) :
                QLabel(parent)
        {
        }

        bool DragAndDropLabel::event (QEvent* event)
        {
            if (event->type()==QEvent::ToolTip) {
                event->setAccepted(true);
                return true;
            }
            return QWidget::event(event);
        }

        void DragAndDropLabel::dragEnterEvent (QDragEnterEvent* event)
        {
            cout << event->mimeData()->formats().join(",") << endl;
            if ((event->mimeData()->hasFormat("text/uri-list") ||
                event->mimeData()->hasFormat("text/plain") ||
                event->mimeData()->hasFormat("text/x-moz-url") ||
                event->mimeData()->hasImage()
                )
                &&
                ( event->proposedAction()==Qt::CopyAction ||
                  event->proposedAction()==Qt::MoveAction)) {
                event->acceptProposedAction();
            }
        }

        void DragAndDropLabel::dropEvent (QDropEvent* event)
        {
            QList<QUrl> urls;
            QStringList lines;

            // Text data? Thread as Url List

            if (event->mimeData()->hasUrls()) {
                urls = event->mimeData()->urls();
            }
            else if (event->mimeData()->hasText()) {
                lines = event->mimeData()->text().split(QRegExp("\r?\n"));
            }
            else if (event->mimeData()->hasFormat("text/x-moz-url")) {
                QByteArray buffer = event->mimeData()->data("text/x-moz-url");
                lines = QString::fromUtf16((const ushort*)buffer.data(),buffer.size()).split(QRegExp("\r?\n"));
            }

            if (!lines.empty()) {
                foreach (QString line, lines) {
                    urls.append(QUrl(line));
                }
            }

            if (!urls.empty()) {
                emit urlsToLoad (urls);
                event->acceptProposedAction();
                return;
            }

            // Found image data
            if (event->mimeData()->hasImage()) {
                const QImage image = qvariant_cast<QImage>(event->mimeData()->imageData());
                emit imageToLoad (image);
                event->acceptProposedAction();
                return;
            }
        }
    }
}
