/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*************************************************************************/

#ifndef SIMPLESELECTOR_H
#define SIMPLESELECTOR_H

#include "model/tagitem.h"

#include <QWidget>
#include <QModelIndex>

#include "xquerycoversearcher.h"

namespace Ui {
    class SimpleSelector;
}

class QStandardItemModel;
class QListView;
class QLabel;

class CoverSelectorModel;

namespace Texel {
    namespace CoverSearch {

        class SimpleSelector;

        namespace Internal {
            class ListController: public QObject
            {
                Q_OBJECT
            public:
                ListController (XQueryCoverSearcher* searcher, QListView* view, CoverSelectorModel* model);
                virtual ~ListController ();

            public slots:
                void onSearchStart ();
                void onProgress (int p);
                void onSearchEnd ();

            signals:

            protected:
                virtual bool eventFilter(QObject *obj, QEvent *event);

            private:
                QWidget *createOverlay();
                void centerContainer (QWidget *container, const QSize &viewSize);

                CoverSelectorModel* m_Model;
                XQueryCoverSearcher* m_Searcher;
                QListView* m_View;

                QWidget* m_OverlayLabel;
                QLabel* m_ProgressLabel;
            };
        }

        class SimpleSelector : public QWidget
        {
            Q_OBJECT

        public:
            struct SearchArgs
            {
                SearchArgs ();
                SearchArgs (const QString& artist, const QString& album);

                QString artist;
                QString album;
                QImage cover;

                bool operator< (const SearchArgs& other) const;
            };

            explicit SimpleSelector(const QList<XQueryCoverSearcher*> searchers, const QList<SearchArgs>& args);
            ~SimpleSelector();

            int getCurrentAlbumRow () const;

            static void search (const TagItem::List& items, QObject* receiver, const char* slot);

        protected:
            void changeEvent(QEvent *e);

        signals:
            void newCover (const QString& artist, const QString& album, const QImage& cover);

        public slots:
            void onResearchClicked ();

        protected slots:
            void onSearchStart ();
            void onSearchEnd ();
            void onAlbumSelection (const QModelIndex& i);
            void onAlbumSelection (const QString& artist, const QString& album);
            void onNextAlbum ();
            void onPreviousAlbum ();
            void onError (const QString& message);
            void onErrorLink (const QString& link);
            void onSelectedCover ();
            void onDoubleClick (const QModelIndex& index);

        private:
            CoverSelectorModel* createListView (const QString& name, CoverSelectorModel* master, XQueryCoverSearcher* searcher);

            Ui::SimpleSelector *ui;
            QList<XQueryCoverSearcher*> m_Searchers;
            QList<SearchArgs> m_Args;
            CoverSelectorModel* m_Master;
            Internal::ListController* m_MasterController;

            int m_SelectedRow;
            int m_OpenSearches;
        };

        inline SimpleSelector::SearchArgs::SearchArgs ()
        {
        }

        inline SimpleSelector::SearchArgs::SearchArgs (const QString& artist_, const QString& album_)
            : artist(artist_), album(album_)
        {
        }

        inline bool SimpleSelector::SearchArgs::operator< (const SimpleSelector::SearchArgs& other) const
        {
            if (artist==other.artist) {
                return album<other.album;
            }
            return artist<other.artist;
        }

        inline int SimpleSelector::getCurrentAlbumRow () const
        {
            return m_SelectedRow;
        }

    }
}

#endif // SIMPLESELECTOR_H
