/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*************************************************************************/

#include "toolbox.h"

#include "simpleselector.h"
#include "xquerycoversearcher.h"

#include "options/preferences.h"
#include "model/tagmodelproxy.h"
#include "model/tagextension.h"
#include "util/loghook.h"

#include <QByteArray>
#include <QFile>
#include <QFileInfo>
#include <QObject>
#include <QCoreApplication>

#include <kio/job.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include "aid.h"

using namespace std;

namespace Texel
{
    namespace CoverSearch {

        ToolBox::ToolBox()
        {
        }

        class FilterForCoverJob::Private: public QThread
        {
            Q_OBJECT

        public:
            volatile bool m_Stopped;

            Private (FilterForCoverJob* job, const TagModelProxy* model_, bool includeCover_) :
                    m_Job(job),
                    model(model_),
                    includeCover(includeCover_)
            {
            }

            virtual ~Private ()
            {
            }

        protected:
            virtual void run ()
            {
                m_Stopped=false;
                emit m_Job->description(m_Job,
                                        i18n("Searching for images in all files (loading of each file necessary!)"));

                m_foundCovers.clear();
                for (int i=0;i<model->rowCount();i++) {
                    TagItem* item = model->getRow(i);
                    if (item->supportsCoverImages()) {
                        TagExtension* extension=item->getExtensions();
                        if (extension!=0l && (extension->images().size()==0)!=includeCover) {
                            m_foundCovers << item;
                        }
                    }
                    m_Job->emitPercent(i,model->rowCount());

                    if (m_Stopped) break;
                }
            }

        public:
            FilterForCoverJob* m_Job;
            const TagModelProxy* model;
            bool includeCover;
            TagItem::List m_foundCovers;
        };

        TagItem::List FilterForCoverJob::getFoundCovers () const
        {
            return m_Thread->m_foundCovers;
        }

        FilterForCoverJob::FilterForCoverJob (const TagModelProxy* model_, bool includeCover_):
                KJob(0l),
                m_Thread (new Private(this,model_,includeCover_))
        {
            setCapabilities(KJob::Killable);
            connect (m_Thread, SIGNAL(finished()), this, SLOT(onFinished()));
        }

        FilterForCoverJob::~FilterForCoverJob()
        {
            if (m_Thread->isRunning()) {
                kill();
            }
            delete m_Thread;
        }

        void FilterForCoverJob::onFinished ()
        {
            emitResult();
        }

        void FilterForCoverJob::start ()
        {
            m_Thread->start();
        }

        bool FilterForCoverJob::doKill ()
        {
            if ( m_Thread->isRunning() ) {
                 m_Thread->m_Stopped = true;
                 m_Thread->wait();
                 return true;
             }
             else {
                 return false;
             }
        }

        FilterForCoverJob* ToolBox::filterForCover (const TagModelProxy* model, bool includeCover) const
        {
            return new FilterForCoverJob(model,includeCover);
        }

        namespace Internal_nouse {

            SearchArgsEnricher::SearchArgsEnricher ()
                : QObject(0l),
                m_IsProcessed(false)
            {
            }

            SearchArgsEnricher::~SearchArgsEnricher ()
            {
            }

            /*void SearchArgsEnricher::connect (Texel::CoverSearch::CoverSearcher* searcher, QObject* receiver, const char* slot)
            {
                QObject::connect (searcher, SIGNAL(coverSelected(QImage)), this, SLOT(received(QImage)));
                QObject::connect (this, SIGNAL(foundCover(QString,QString,QImage)), receiver, slot);
            }*/

            void SearchArgsEnricher::received (const QImage& cover)
            {
                m_IsProcessed=true;
                emit foundCover (m_Artist,m_Album, cover);
            }

        }


    }
}

#include "toolbox.moc"
