/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*************************************************************************/

#ifndef TOOLBOX_H
#define TOOLBOX_H

#include <QImage>
#include <kurl.h>

#include "simpleselector.h"
#include "model/tagitem.h"

#include <kjob.h>

class QByteArray;

namespace Texel
{
    class MessageModel;
    class TagModelProxy;
    class ProgressProxy;

    namespace CoverSearch
    {
       class CoverSearcher;
       class CoverSource;
    }

    namespace CoverSearch {


        class FilterForCoverJob: public KJob
        {
            Q_OBJECT
        public:
            FilterForCoverJob (const TagModelProxy* model_, bool includeCover_);
            virtual ~FilterForCoverJob ();
            virtual void start ();

            TagItem::List getFoundCovers () const;

        protected:
            virtual bool doKill ();

        private slots:
            void onFinished ();

        private:
            class Private;
            Private* m_Thread;
            friend class Private;
        };

        class ToolBox
        {
        public:
            ToolBox();

            FilterForCoverJob* filterForCover (const TagModelProxy* model, bool includeCover) const;

        private:
        };

        namespace Internal_nouse {

            class SearchArgsEnricher: public QObject
            {
                Q_OBJECT
            public:
                SearchArgsEnricher ();
                virtual ~SearchArgsEnricher ();

                void setSearchArgs (const QString& artist, const QString& album);
                //void connect (Texel::CoverSearch::CoverSearcher* searcher, QObject* receiver, const char* slot);

                bool isProcessed () const;
                void resetProcessFlag();

            private slots:
                void received (const QImage& cover);

            signals:
                void foundCover (const QString& artist, const QString& album, const QImage cover);

            private:
                QString m_Album;
                QString m_Artist;
                bool m_IsProcessed;
            };

            inline void SearchArgsEnricher::setSearchArgs (const QString& artist, const QString& album)
            {
                m_Album=album;
                m_Artist=artist;
            }

            inline bool SearchArgsEnricher::isProcessed () const
            {
                return m_IsProcessed;
            }

            inline void SearchArgsEnricher::resetProcessFlag()
            {
                m_IsProcessed=false;
            }

        }

    }
}

#endif // TOOLBOX_H
