/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef XQUERYCOVERSEARCHER_H
#define XQUERYCOVERSEARCHER_H

#include <QString>
#include <QList>
#include <QMap>

#include "cover.h"
#include "xquerysearcher.h"

class QProgressBar;

namespace Texel {

namespace CoverSearch {

class XQueryCoverSearcher : public XQuerySearcher
{
    Q_OBJECT
public:
    static QList<XQueryCoverSearcher*> getConfiguredSearchers (bool activeOnly, QObject* parent);

    explicit XQueryCoverSearcher(QObject *parent = 0);

    void search (const QString& artist, const QString& album);
    void startExtraction (Cover* cover);

    void setPreviewUrlTemplate (const QString& url);
    QString previewUrlTemplate () const;

    void setSquareImagesOnly (bool only);
    void setMaxPreviews (int prevs);

    QUrl getSpecificUrl (const QString &artist, const QString &album) const;

protected:
    virtual void doAddVariablesToQuery (SimpleXMLFromPage *loader);
    virtual void parsePreviewResultXML (QXmlStreamReader* reader, QAbstractItemModel* model) throw (xml_parse_error);

    virtual void doSaveLocalConfig (QXmlStreamWriter& writer);
    virtual void doReadLocalConfig (QXmlStreamReader& reader);

private:
    void parseCoverSource_result (QXmlStreamReader* reader, QAbstractItemModel* model) throw (xml_parse_error);
    void parseCoverSource_cover (QXmlStreamReader* reader, QAbstractItemModel* model) throw (xml_parse_error);

    bool m_SquareImagesOnly;
    int m_MaxPreviews;

    // Preview retrieving
    QString m_PreviewUrlTemplate;

    // Temporary
    QString m_Artist;
    QString m_Album;
};

inline QString XQueryCoverSearcher::previewUrlTemplate () const
{
    return m_PreviewUrlTemplate;
}

inline void XQueryCoverSearcher::setSquareImagesOnly(bool only)
{
    m_SquareImagesOnly=only;
}

inline void XQueryCoverSearcher::setMaxPreviews (int prevs)
{
    m_MaxPreviews=prevs;
}

}

}

#endif // XQUERYCOVERSEARCHER_H
