/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef M3ULIST_H
#define M3ULIST_H

#include <QObject>
#include <QString>
#include <QList>
#include <QExplicitlySharedDataPointer>

class QFile;
class QIODevice;

namespace Texel {

    class TagItem;
    class TagModel;

    namespace M3U {

        class List
        {
        public:
            struct Track
            {
                Track ()
                    : extended(false), dirty(false), length(-1)
                {
                }

                QString getFilename () const;

                bool extended;
                bool dirty;
                QString title;
                QString filename;
                int length;
                TagItem* item;
            };

            Track createTrack (TagItem* item);

            List (TagModel* model=0l);
            List (const List& other);
            ~List ();

            void removeFromTags ();

            void load (const QString& filename);
            void save (const QString& filename="");

            void load (QIODevice* device, const QString& workingDir, bool expandFileNames);
            void save (QIODevice* device, bool enforceAbsolutePathes) const;

            void setTracks (const QList<TagItem*>& tracks);
            void setTracks (const QList<Track>& tracks);
            void removeTrack (TagItem* track);
            void removeTrack (int pos);
            void insertTrack (int pos, TagItem* track);

            void setFileName (const QString& filename);
            QString getFileName () const;
            QString getName () const;
            QString getLoadFileName () const;

            void incAlteredTracks ();
            void decAlteredTracks ();
            int alteredTracks () const;

            int size () const;
            bool isExtended () const;
            bool isDirty () const;
            bool wasSaved () const;
            bool isLocal () const;
            bool isDirty (int pos) const;
            void clear();

            quint32 getUniqId () const;

            QString getName (int index) const;
            QString getFileName (int index) const;
            TagItem* getItem (int index) const;
            Track get (int index) const;
            void move (int from, int to);
            void remove (int pos);
            void set (int pos, const Track& track);
            QList<TagItem*> getItems () const;
            QList<Track> getTracks () const;

            static List createPlaylist (TagModel* model, quint32 id, const QString& filename, const QString& loadedFilename,
                                        int alteredTracks, bool extended, bool dirty, bool local, bool saved);
            static List createPlaylist (const QList<TagItem*>& tracks);
            static List createPlaylist (TagModel* model, QIODevice* device, const QString& filename);

            bool operator== (const List& other) const;
            const List& operator= (const List& other);

        protected:
            void removeFromItem (TagItem* track);
            void insertIntoItem (TagItem* track);

        private:
            TagItem* mapFilename (const QString& filename);
            void writeLine (QIODevice* file, const QString& line) const;

            // Use explicitely shared data to avoid problems with deletion of the lists -
            // owner is not really clear as so many references may exist
            class ListData;
            QExplicitlySharedDataPointer<ListData> p;
        };

    }
}

#endif // M3ULIST_H
