/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PLAYLISTMODEL_H
#define PLAYLISTMODEL_H

#include <QAbstractItemModel>
#include <QList>

#include "playlistgroup.h"
#include "model/tagitem.h"

namespace Texel {

    class TagModelProxy;
    class TagModel;

    class PlaylistModel : public QAbstractItemModel, protected PlaylistGroup
    {
        Q_OBJECT
    public:
        explicit PlaylistModel (TagModel* model, QObject *parent = 0);

        virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
        virtual QModelIndex parent(const QModelIndex &child) const;
        virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
        virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
        virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
        virtual bool setData (const QModelIndex &index, const QVariant &value, int role);

        void addPlaylist (M3U::List playlist);
        void removePlaylist (int pos, bool deleteOnSave);
        M3U::List getPlaylist (int row) const;

        void removeTrack (QModelIndex idx);
        void insertTracks (const QModelIndex& idx, TagItem::List items);

        TagItem::List getTracks (QModelIndex idx) const;
        QString getFilename (int row) const;

        void deleteAll ();

        void updateChanged ();

        void decDirtyCounter (const PlaylistGroup& playlists);
        void incDirtyCounter (const PlaylistGroup& playlists);

        virtual Qt::ItemFlags flags (const QModelIndex& ) const;

        virtual bool dropMimeData (const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent);
        virtual QMimeData* mimeData (const QModelIndexList& indexes) const;
        virtual QStringList mimeTypes () const;
        virtual Qt::DropActions supportedDropActions() const;

        void removeRow (int row);

        static void sortIndexList (QModelIndexList& list);

    signals:

    public slots:

    private:
        TagModel* m_Model;
        unsigned m_UntitledCounter;
        QList<QString> m_PlaylistsToDelete;
    };

    inline QString PlaylistModel::getFilename(int row) const
    {
        return m_Playlists[row].getFileName();
    }

    inline M3U::List PlaylistModel::getPlaylist (int row) const
    {
        if (row>=0 && row<m_Playlists.size()) {
            return m_Playlists.at(row);
        }
        return m_Playlists.front();
    }

}

#endif // PLAYLISTMODEL_H
