/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef CHECKSERVICEARTIST_H
#define CHECKSERVICEARTIST_H

#include <QThread>
#include <QStringList>
#include <QList>
#include <QPair>
#include <QVariant>

class PersistentCache;

namespace Texel
{

namespace CheckService
{

class ArtistQuery: public QObject
{
Q_OBJECT
public:
    ArtistQuery (QObject* parent);
    virtual ~ArtistQuery ();

    QPair<QStringList,QStringList> getCacheInformation () const;

public slots:
    void lookup (const QVariant& ref, const QString& artist);
    void execLookup ();

signals:
    void result (const QVariant& ref, const QStringList& results);

protected:
    virtual QStringList doLookup (const QString& artist)=0;

private:
    QVariant m_Ref;
    QString m_Artist;

    QList<QPair<QVariant,QString> > m_Queue;
};

class CheckArtist: public QThread
{
public:
    CheckArtist(ArtistQuery* query);
    ~CheckArtist();

    virtual void run ();

    void setCacheParameters (const QString& name, int itemsInMemory, int maxFileSize);
    void attach (QObject* obj, const char* lookupSignal, const char* resultSlot);

    static void msleep (unsigned long millis);

    QPair<QStringList,QStringList> getCacheInformation () const;

private:
    ArtistQuery* m_Query;

    QString m_CacheFile;
    int m_ItemsInMemory;
    int m_MaxFileSize;
};

inline void CheckArtist::setCacheParameters (const QString& name, int itemsInMemory, int maxFileSize)
{
    m_CacheFile=name;
    m_ItemsInMemory=itemsInMemory;
    m_MaxFileSize=maxFileSize;
}

inline void CheckArtist::msleep(unsigned long millis)
{
    QThread::msleep(millis);
}

}

}

#endif // CHECKSERVICEARTIST_H
