/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef CDENTRY_H
#define CDENTRY_H

#include <QList>
#include <QSet>
#include <QString>
#include <QDataStream>
#include <model/tagitem.h>

class QIODevice;

namespace Texel {

    struct CDReference
    {
        typedef QList<CDReference> List;
        typedef QSet<CDReference> Set;

        CDReference ()
            : genre(0), discid(0), title()
        {
        }

        CDReference (const QString& g, quint32 id, const QString& t=QString());

        CDReference (quint16 g, quint32 id, const QString& t=QString())
            : genre(g), discid(id), title(t)
        {
        }

        bool operator !=(const CDReference& other) const
        {
            return discid!=other.discid || genre!=other.genre;
        }

        bool operator ==(const CDReference& other) const
        {
            return discid==other.discid && genre==other.genre;
        }

        bool isEmpty() const
        {
            return discid==0;
        }

        const QString& getGenre () const;

        quint16 genre;
        quint32 discid;
        QString title;

    private:
        static QList<QString> categories;
        static bool _init;
        static void init();
    };

    struct CDEntry
    {
        CDEntry ()
            : year(0),
            sampler(false)
        {
        }

        QString title;
        QString genre;
        quint16 year;

        static CDEntry* parse (QIODevice* buffer);

        void mapContent (const TagItem::List& items, bool unsetOnly);
        void mapToItem (TagItem* const item, int track, bool unsetOnly);

        struct Track
        {
            Track (int _runtime=0)
                : runtime(_runtime)
            {
            }

            const QString& col (int column) const;

            QString title;
            QString artist;
            int runtime;
        };

        QList<Track> tracks;
        bool sampler;
    };

    inline uint qHash (const Texel::CDReference& ref)
    {
        return ref.discid|((ref.genre&0b11)<<6)|(((ref.genre>>2)&0b11)<<22);
    }
}

QDataStream& operator<< (QDataStream& stream, const Texel::CDEntry& ref);
QDataStream& operator<< (QDataStream& stream, const Texel::CDEntry::Track& track);
QDataStream& operator<< (QDataStream& stream, const Texel::CDReference& track);
QDataStream& operator>> (QDataStream& stream, Texel::CDEntry& ref);
QDataStream& operator>> (QDataStream& stream, Texel::CDEntry::Track& track);
QDataStream& operator>> (QDataStream& stream, Texel::CDReference& track);

#endif // CDENTRY_H
