/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef DETAILSMENU_H
#define DETAILSMENU_H

#include "freedbaccess.h"
#include "model/tagitem.h"

#include <QMenu>
#include <QUrl>

class QTextBrowser;
class QTextTable;
class QTextBlockFormat;

namespace Texel {

    namespace FreeDB {

        class QueryMenu;

        class DetailsMenu : public QMenu
        {
            Q_OBJECT
        public:
            explicit DetailsMenu (FreeDBAccess* access, const CDReference& ref, QueryMenu* parent = 0);

        signals:

        public slots:
            void onShow ();
            void onResult (const CDEntry& entry);
            void onError (const QString& description);

            void takeAll ();
            void takeUnfilled ();
            void takeTrackNumbers ();
            void takeFromLink (const QUrl& link);
            void setToolTip (const QUrl& link);

        private:
            void updateCell (QTextBrowser* browser, const QString& value, TagItem* item, int col, int x);
            void insertCell (QTextTable* table, const QTextBlockFormat& blockformat, const QString& text, bool asLink, int row, int col, int kind);

            QTextBrowser* renderTable (const CDEntry& entry);
            QUrl createUrl (int col,int x);
            void parseUrl (const QUrl& url, int& col, int& x);

            FreeDBAccess* m_Accessor;
            CDReference m_Reference;
            int status;

            CDEntry m_Entry;
            TagItem::List m_MappedRows;
            QTextTable* m_Table;
        };
    }
}

#endif // DETAILSMENU_H
