/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef FREEDBACCESS_H
#define FREEDBACCESS_H

#include <QPair>
#include <QList>
#include <QStringList>

#include "func/metatagservices/freedb/cdentry.h"

class QByteAray;
class KJob;

namespace Texel {
    namespace FreeDB {

        /**
  This class encapsulated the reverse lookup functionality for free db. Idea is that in many cases
  the length of the songs differs from the toc based lengthes by a second (due to rounding issues).
  Therefore this class uses a continues index to check for potential discid candidates. As further
  checks the length and track count is taken as well as te length of the first four tracks.

  This class provides only access to the HTTP interface of freedb, as this is the preferred method
  suggested by the freedb team. It is not using the kcddb access library as it does need lowlevel
  access to the freedb interface (dealing with discids and not only TOCs).
  */

        class FreeDBAccess: public QObject
        {
            Q_OBJECT

        public:
            FreeDBAccess();
            virtual ~FreeDBAccess();

            void lookup (const QList<int>& toc, bool sorted);

            void readDiscContent (const CDReference& reference);

            static bool existIndex ();

        protected:
            QString hello () const;
            QString proto () const;
            KJob* executeHttpCommand (const QString& cmd);
            QString queryString (quint32 trackid, const QList<int>& toc) const;
            QString readString (const CDReference& reference) const;

        signals:
            /** Is send when a error occured on requesting a freedb cd.
      */
            void queryError (const QString description, const QString& details);
            void potentialMatches (const CDReference::List& list);
            void finishedQuery ();

            void detailsMatch (const CDEntry& entry);
            void detailsError (const QString description, const QString& details);

        protected slots:
            void onQueryDiscIdResponse (KJob *job);
            void onReadDiscContentResponse (KJob *job);

        private:
            static const QChar s_CddbSeparator;
            static bool s_RegisteredMetaType;

            CDReference::List searchDiscId (quint64 footprint);

            quint32 calcDiscId (const QList<int>& toc);
            void removeDiscidFromList (quint32 discid);
            void queryDiscid ();

            CDReference m_CurrentDiscId;
            CDReference m_CurrentDiscIdForEntry;

            // Stores a list of discids which fits to the given toc. the entries are sorted,
            // most relevant at the beginning of the list.
            CDReference::List m_LookedUpCDReferences;

            int m_QueryCounter;
            QList<int> m_TOC;
        };
    }
}

#endif // FREEDBACCESS_H
