/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    $Id$

*************************************************************************/

#ifndef INDEXDOWNLOADER_H
#define INDEXDOWNLOADER_H

#include <QObject>
#include <zlib.h>

class KJob;
namespace KIO { class Job; }

class QByteArray;
class QFile;
class KUrl;

namespace Texel {

    namespace FreeDB {

    class TrackerWithWarning;

    /**
      Controler to download index from sourceforge, decompress it in memory and save result
      to the defined index location. The controler can be started and forgotten about; it
      deletes itself after successful download. User feedback is given by providing a status
      bar tracker which shows up in the KDE status bar.

      Memory decompression is done via direct zlib access as existing methods provide no method
      to decompress memory and save to a file. All other solutions would include a temporary
      file, which does usually increase complexity: Ensure enough space, deletion on crashes,
      etc.
      */
    class IndexDownloader: public QObject
    {
        Q_OBJECT

    public:
        IndexDownloader(QObject* parent);
        virtual ~IndexDownloader();

    private slots:
        void onFinished (KJob* job);
        void onCancel (KJob* job);
        void onData (KIO::Job* job, const QByteArray& buffer);

    private:
        TrackerWithWarning* m_Tracker;
        /// Destination where the index is decompressed to
        QFile* m_Dest;
        /// Decompression state
        z_stream* strm;

        bool m_TemporaryFile;
    };
}
}

#endif // INDEXDOWNLOADER_H
