#ifndef QUERYMENU_H
#define QUERYMENU_H

#include <QMenu>
#include <QList>
#include <QSet>

#include "freedbaccess.h"
#include "model/tagitem.h"

namespace Texel {

    class TagModelProxy;

    namespace FreeDB {

        class QueryMenu : public QMenu
        {
            Q_OBJECT
        public:
            explicit QueryMenu(TagModelProxy* model, const TagItem::List& selectedRows, QWidget *parent = 0);
            virtual ~QueryMenu();

            TagItem::List getSelectedRows () const;
            TagModelProxy* model() const;

            TagItem::List reorderByCDEntry (const CDEntry& entry);

            bool isSorted () const;

        signals:

        private slots:
            void onShow ();
            void onResults (const CDReference::List& references);
            void onError (const QString& description);
            void onFinished ();
            void downloadIndex ();

        private:
            void errorToLabel (const QString& description);

            TagItem::List m_SelectedRows;
            FreeDBAccess* m_Accessor;
            TagModelProxy* const m_Model;
            int status;
            bool m_ResultsAvailable;
            bool m_Sorted;
            QSet<CDReference> m_Displayed;
        };

        inline TagItem::List QueryMenu::getSelectedRows() const
        {
            return m_SelectedRows;
        }

        inline TagModelProxy* QueryMenu::model() const
        {
            return m_Model;
        }

        inline bool QueryMenu::isSorted() const
        {
            return m_Sorted;
        }
    }
}

#endif // QUERYMENU_H
