/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "musicbrainz.h"
#include <musicbrainz3/webservice.h>
#include <musicbrainz3/query.h>

#include "util/loghook.h"

#include <klocale.h>

using namespace MusicBrainz;

namespace Texel {

namespace CheckService {

MusicBrainzQuery::MusicBrainzQuery (QObject* parent)
    : ArtistQuery(parent),
      w(new WebService()),
      query(new Query(w))
{
    w->setUserName("****");
    w->setPassword("****");
}

MusicBrainzQuery::~MusicBrainzQuery()
{
    delete query;
    delete w;
}

QStringList MusicBrainzQuery::doLookup (const QString& artist)
{
    QStringList r;
    try {
        ArtistFilter filter;
        filter.name(artist.toLocal8Bit().data());
        filter.limit(10);
        const int waitTime=measure.elapsed();
        if (waitTime<1100) CheckArtist::msleep(1100-waitTime);
        measure.start();
        ArtistResultList list = query->getArtists(&filter);
        LogHook::instance()->message(i18n("Look up artist %1 on musibrainz",artist),0l);

        for (unsigned i=0;i<list.size();i++) {
            r << QString::fromUtf8(list[i]->getArtist()->getName().c_str());
            delete list[i];
        }
    } catch (...) {}

    return r;
}

}

}

#include "musicbrainz.moc"
