/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "musicbrainztagsuggestion.h"

#include <model/tagitem.h>
#include "util/loghook.h"

#include <musicbrainz3/webservice.h>
#include <musicbrainz3/query.h>

namespace Texel {

MusicBrainzTagSuggestion::MusicBrainzTagSuggestion(const QString& name, TagItem* referer)
    : TagSuggestion(name,referer)
{
}

QString MusicBrainzTagSuggestion::typeName() const
{
    return "Musicbrainz tag suggestion";
}

TagItemType::Type MusicBrainzTagSuggestion::type() const
{
    return TagItemType::MusicBrainzTagSuggestion;
}

QString MusicBrainzTagSuggestion::suggestFilenameExtension() const
{
    return "";
}

void MusicBrainzTagSuggestion::saveMetaTag() throw (TagItem::IOError)
{
    // Nothing to save
}

TagItem* MusicBrainzTagSuggestion::createTagItemFromPUID (TagItem* referer, const QString& puid)
{
    MusicBrainz::WebService w;
    MusicBrainz::Query query(&w);
    MusicBrainz::TrackFilter filter;
    filter.puid(puid.toLatin1().data());

    try {
        MusicBrainz::TrackResultList list=query.getTracks(&filter);
        if (list.size()>0) {
            MusicBrainzTagSuggestion* tag = new MusicBrainzTagSuggestion(i18n("From Musicbrainz"), referer);
            tag->setData(ColumnManager::Artist, QString(list[0]->getTrack()->getArtist()->getName().c_str()));
            tag->setData(ColumnManager::Title, QString(list[0]->getTrack()->getTitle().c_str()));
            tag->m_Puid=puid;
            return tag;
        }
        else {
            LogHook::instance()->warning (i18n("No entry found for PUID %1",puid),referer);
            return 0l;
        }
    }
    catch (MusicBrainz::ConnectionError)
    {
        LogHook::instance()->error(i18n("Could not connect to musicbrainz server"),referer);
    }
    catch (MusicBrainz::RequestError)
    {
        LogHook::instance()->error(i18n("PUID %1 is in invalid format",puid),referer);
    }
    catch (...)
    {
        LogHook::instance()->error(i18n("Unidentified error while requesting musicbrainz information"),referer);
    }

    return 0l;
}

}
