/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "parserproblem.h"
#include <klocale.h>

using namespace Texel;

ParserProblem::ParserProblem (const QString& problem)
: InputProblem(),
  m_Problem(problem)
{
}

ParserProblem::~ParserProblem()
{
}

QString ParserProblem::getText() const
{
   return i18nc("Displays an error detected while parsing","Parser error: %1",m_Problem);
}

void ParserProblem::setSuggestion (const QString& input, const QString& suggestion, unsigned begin, unsigned end)
{
   m_Suggestion=input;
   m_Suggestion.replace(begin,end-begin,suggestion);
}

QList<QAction*> ParserProblem::getActions ()
{
   if (m_Problem.isEmpty()) return QList<QAction*>();
   QList<QAction*> actions;
   QAction* action=new QAction(i18nc("Button command; when pressed an error is automatically resolved.","Fix it"),this);
   connect (action, SIGNAL(triggered()), this, SLOT(fixit()));
   return actions << action;
}

void ParserProblem::fixit ()
{
   setData (m_Suggestion);
}

void ParserProblem::setLocator (const QString locator)
{
   m_Locator=locator;
}

QString ParserProblem::locator() const
{
   return m_Locator;
}

bool ParserProblem::isFormularProblem() const
{
    return true;
}

#include "parserproblem.moc"
