/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "moveindicatortable.h"

#include <QPainter>
#include <QDragMoveEvent>
#include <QLabel>

#include "aid.h"

using namespace std;

namespace Texel {

    MoveIndicatorTable::MoveIndicatorTable(QWidget* parent)
    : QTableView(parent),
      w(0l),
      m_InDrag(false),
      m_Offset()
    {
        setDragDropMode(QAbstractItemView::InternalMove);
        setDragEnabled(true);
        viewport()->setAcceptDrops(true);
    }

    MoveIndicatorTable::~MoveIndicatorTable()
    {
        delete w;
    }

    void MoveIndicatorTable::resizeEvent (QResizeEvent* event)
    {
        QAbstractItemView::resizeEvent(event);
        if (w!=0l) {
            w->resize(viewport()->width(),2);
        }
    }

    void MoveIndicatorTable::drawInsertIndicator (int h)
    {
        if (w!=0l) {
            w->move(QPoint(0,h)+m_Offset);
        }
    }

    void MoveIndicatorTable::dragMoveEvent (QDragMoveEvent* event)
    {
        if (!m_InDrag) {
            if (w==0l) {
                w=new QWidget(this);
                w->resize(viewport()->width(),2);
                w->setAutoFillBackground(true);
                QPalette p=palette();
                p.setBrush(QPalette::Window,QBrush(QColor(240,0,0)));
                w->setPalette(p);
            }
            w->show();
            m_InDrag=true;

            m_Offset=viewport()->mapToGlobal(QPoint(0,0))-mapToGlobal(QPoint(0,0));
        }

        QAbstractItemView::dragMoveEvent (event);

        if (event->mimeData()->hasFormat("application/enqueue/index")) {
            const QModelIndex idx=indexAt(event->pos());
            if (idx.isValid()) {
                const QRect r = visualRect(idx);
                drawInsertIndicator (r.top());
            }
            else if (dropIndicatorPosition()==QAbstractItemView::OnViewport) {
                const QRect r = visualRect(model()->index(model()->rowCount()-1,0));
                drawInsertIndicator (r.bottom());
            }
        }
    }

    void MoveIndicatorTable::dragLeaveEvent (QDragLeaveEvent* event)
    {
        QAbstractItemView::dragLeaveEvent(event);
        if (w!=0l) w->hide();
    }

    void MoveIndicatorTable::dragEnterEvent (QDragEnterEvent* event)
    {
        QAbstractItemView::dragEnterEvent(event);
        if (w!=0l) w->show();
    }

    void MoveIndicatorTable::dropEvent (QDropEvent * event)
    {
        QAbstractItemView::dropEvent (event);
        w->hide();
        m_InDrag=false;
        selectionModel()->clearSelection();
    }

}

#include "moveindicatortable.moc"
