/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "phraseitem.h"

#include "model/tagitem.h"
#include "model/columnmanager.h"
#include "model/texeltagmodel.h"
#include "aid.h"

#include <QMenu>
#include <QStack>
#include <QModelIndex>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>

using namespace std;

namespace Texel {

const int PhraseItem::Offset=3;

PhraseItem::PhraseItem()
: m_Phrases(ColumnManager::instance()->getEditableColumns()),
  m_Problems(ColumnManager::instance()->getEditableColumns()+Offset)
{
   for (int i=0;i<m_Problems.size();i++) m_Problems[i]=0l;
}

PhraseItem::~PhraseItem()
{
    for (int i=0;i<m_Problems.size();i++) delete m_Problems[i];
}

void PhraseItem::addToMenu (QMenu* menu,const QObject* receiver, const char* member)
{
    // Replace double // by another token to allow correct splitting, correct it afterwards
    QStringList path = QString(m_Name).replace("//","§").split("/").replaceInStrings("§","/");

    QMenu* current=menu;
    for (int i=0;i<path.size()-1;i++) {
        QList<QAction*> entries=current->actions();
        QMenu* child=0l;
        foreach (QAction* entry, entries) {
            if (entry->text()==path[i]) {
                child=entry->menu();
                break;
            }
        }

        if (child!=0l) {
            current=child;
        }
        else {
            current=current->addMenu(path[i]);
        }
    }

    QAction* action=current->addAction(path[path.size()-1],receiver,member,m_HotKey);
    action->setData(QVariant::fromValue(this));
    action->setToolTip(m_Description);
    action->setStatusTip(m_Description);
}

void PhraseItem::execute (TagItem* item)
{
    for (int i=0;i<m_Phrases.size();i++) {
        if (!m_Phrases[i].isEmpty()) {
            const QModelIndex index=item->getModel()->index(item->getRow(),ColumnManager::instance()->mapFromEditableColumn(i));
            item->getModel()->setData(index,QVariant(m_Phrases[i]));
        }
    }
}

void PhraseItem::setName (const QString& name)
{
    m_Name=name;
}

void PhraseItem::setDesc (const QString& desc)
{
    m_Description=desc;
}

void PhraseItem::setData (int column, const QString& value)
{
    if (column>=0 && column<m_Phrases.size()) {
        delete m_Problems[column+Offset];
        m_Problems[column+Offset]=0l;
        m_Phrases[column]=value;
        try {
            eval(value,QVariant());
        }
        catch (InputProblem* ip) {
           setProblem(column+2,ip);
        }
    }
}

QString PhraseItem::getVar (const QString& ,bool , const QVariant& ) const
{
    return KGlobal::staticQString("");
}

bool PhraseItem::setDependency (const QString& name, bool , const QVariant& )
{
    if (ColumnManager::instance()->indexByName(name)!=-1) return true;
    return false;
}

QString PhraseItem::suggestVarName (const QString&) const
{
    return "";
}

QStringList PhraseItem::availableVariables () const
{
    return QStringList();
}

}
