/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PHRASEITEM_H
#define PHRASEITEM_H

#include "func/parser.h"
#include "model/problem.h"

#include <QString>
#include <QVector>
#include <QKeySequence>
#include <QMetaType>

#include <kglobal.h>

#include "aid.h"

class QMenu;

namespace Texel {

class TagItem;

class PhraseItem: public ParserContext
{
public:
    static const int Offset;

    PhraseItem();
    virtual ~PhraseItem();

    void execute (TagItem* item);
    void addToMenu (QMenu* menu, const QObject * receiver, const char * member);
    void removeFromMenu (QMenu* menu);

    const QString& getName () const;
    const QString& getDesc () const;
    void setName (const QString& name);
    void setDesc (const QString& desc);

    QString getHotKeyAsString() const;
    void setHotKeyFromString(const QString& hotkey);
    const QString& data(int column) const;
    void setData (int column, const QString& value);

    bool hasProblem (int i) const;
    const InputProblem* getProblem (int i) const;
    void setProblem (int i, InputProblem* p);

    // From parser context
    virtual QString getVar (const QString& name,bool org, const QVariant& context) const;
    virtual bool setDependency (const QString& name, bool org, const QVariant& context);
    virtual QString suggestVarName (const QString& name) const;

    virtual QStringList availableVariables () const;

private:
    QString m_Name;
    QString m_Description;
    QKeySequence m_HotKey;
    QVector<QString> m_Phrases;
    QVector<InputProblem*> m_Problems;
};

inline bool PhraseItem::hasProblem (int i) const
{
    return m_Problems[i]!=0l;
}

inline void PhraseItem::setProblem (int i, InputProblem* p)
{
    delete m_Problems[i];
    m_Problems[i]=p;
}

inline const InputProblem* PhraseItem::getProblem (int i) const
{
    return m_Problems[i];
}

inline const QString& PhraseItem::getName () const
{
    return m_Name;
}

inline const QString& PhraseItem::getDesc () const
{
    return m_Description;
}

inline QString PhraseItem::getHotKeyAsString() const
{
    return m_HotKey.toString(QKeySequence::PortableText);
}

inline const QString& PhraseItem::data(int column) const
{
    if (column>=0 && column<m_Phrases.size()) {
        return m_Phrases[column];
    }
    return KGlobal::staticQString("");
}

inline void PhraseItem::setHotKeyFromString(const QString& hotkey)
{
    m_HotKey=QKeySequence::fromString(hotkey,QKeySequence::PortableText);
}

}

Q_DECLARE_METATYPE(Texel::PhraseItem*)

#endif // PHRASEITEM_H
