/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PHRASEMODEL_H
#define PHRASEMODEL_H

#include "util/duplicatemap.h"

#include <QAbstractTableModel>
#include <QKeySequence>

class QMenu;
class QXmlStreamReader;
class QXmlStreamWriter;

namespace Texel {

class PhraseItem;

/** The phrasemodel manages all known phrases and does loading and saving automagically.
    It cares for model internal contrains such as
    * unique name of phrase
    * unique hotkey
    * unique formular set (otherwise the phrase is just a duplicate with another name)
  */

class PhraseModel: public QAbstractTableModel
{
Q_OBJECT

public:
    explicit PhraseModel(QObject* parent);
    virtual ~PhraseModel();

    // virtual replacements for readable models
    virtual int rowCount (const QModelIndex& parent = QModelIndex()) const;
    virtual int columnCount (const QModelIndex& parent = QModelIndex()) const;
    virtual QVariant data (const QModelIndex& index, int role = Qt::DisplayRole) const;
    virtual QVariant headerData (int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

    // virtual replacements for writeable models
    virtual bool setData (const QModelIndex& index, const QVariant & value, int role = Qt::EditRole);
    virtual Qt::ItemFlags flags (const QModelIndex& index ) const;

    // virtual replacements for drag and drop support
    virtual bool dropMimeData (const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent);
    virtual Qt::DropActions supportedDropActions() const;
    virtual QStringList mimeTypes () const;
    virtual QMimeData* mimeData (const QModelIndexList& indexes) const;

    int addRow (QAbstractItemModel* model, const QModelIndexList& indexes);
    PhraseItem* addRow ();

    void rebuildMenu (QMenu* menu);

    void read (const QString& filename);
    void write (const QString& filename);

    virtual bool removeRows (int row, int count, const QModelIndex& parent = QModelIndex());

signals:
    void phraseTriggered (PhraseItem* item);

private slots:
    void triggered();

private:
    void write (QXmlStreamWriter& writer);
    void read (QXmlStreamReader& reader);
    void writeItem (QXmlStreamWriter& writer, int row) const;
    void readItem (QXmlStreamReader& reader, int row);

    QList<PhraseItem*> m_Items;
    DuplicateMap::Detector<QString, PhraseItem> m_UniqueNames;
    DuplicateMap::Detector<QKeySequence, PhraseItem> m_UniqueHotKeys;
};

}

#endif // PHRASEMODEL_H
