/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "phraseview.h"
#include "phrasemodel.h"
#include "options/preferences.h"

#include "ui_phraseview.h"

#include <QMenu>
#include <QTableView>
#include <QPushButton>

#include "aid.h"

using namespace std;

namespace Texel
{

    PhraseView::PhraseView(QWidget *parent, PhraseModel* model, QMenu* menuPhrases) :
            KDialog(parent),
            m_ui(new Ui::PhraseView),
            m_PhraseMenu(menuPhrases)
    {
        m_ui->setupUi(mainWidget());
        m_ui->tableEditor->setModel(model);

        connect (this, SIGNAL(cancelClicked()), this, SLOT(onCancel()));
        connect (this, SIGNAL(okClicked()), this, SLOT(onOK()));

        QPushButton* exportButton=new QPushButton("Export ...");
        m_ui->tableEditor->addButton(exportButton);
        QPushButton* importButton=new QPushButton("Import ...");
        m_ui->tableEditor->addButton(importButton);

        setCaption(i18n("Manage phrases"));

        rebuildPhraseMenu();
    }

    PhraseView::~PhraseView()
    {
       delete m_ui;
    }

    void PhraseView::changeEvent(QEvent *e)
    {
        KDialog::changeEvent(e);
        switch (e->type()) {
        case QEvent::LanguageChange:
            m_ui->retranslateUi(this);
            break;
        default:
            break;
        }
    }

    void PhraseView::setManageAction(QAction* action)
    {
        m_ManageAction=action;
    }

    void PhraseView::rebuildPhraseMenu()
    {
        m_PhraseMenu->clear();
        m_PhraseMenu->addAction (i18n("Manage ..."), this, SLOT(edit()));
        m_PhraseMenu->addSeparator();
        model()->rebuildMenu(m_PhraseMenu);
    }

    void PhraseView::onOK()
    {
        const QString filename = Preferences::instance()->getWriteNamePhrasesFile();
        model()->write(filename);

        rebuildPhraseMenu();
        hide();
    }

    void PhraseView::onCancel()
    {
        hide();
    }

    void PhraseView::edit ()
    {
        show();
    }

    PhraseModel* PhraseView::model() const
    {
        return dynamic_cast<PhraseModel*>(m_ui->tableEditor->model());
    }

}

#include "phraseview.moc"
