/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include <func/player/enqueuelist.h>
#include <func/player/enqueuemodel.h>
#include <func/player/player.h>
#include "util/widgets/htmldelegate.h"
#include "view/docks/playerdock.h"

#include <QDockWidget>

#include "model/tagitem.h"

namespace Texel {
    namespace Player {

    EnqueueList::EnqueueList (QWidget* parent)
        : QWidget(parent)
    {
        setupUi(this);
        listView->viewport()->setAcceptDrops(true);
        listView->setAcceptDrops(true);
        listView->setModel(Player::instance()->model());
        listView->setItemDelegate(new HtmlDelegate(listView));

        connect (clearButton, SIGNAL(pressed()), listView->model(), SLOT(clear()));
        connect (closeButton, SIGNAL(pressed()), this, SIGNAL(close()));
        connect (playlistButton, SIGNAL(pressed()), listView->model(), SLOT(createPlaylist()));
        connect (removeButton, SIGNAL(pressed()), this, SLOT(removeSelection()));
        connect (listView, SIGNAL(doubleClicked(const QModelIndex&)), listView->model(), SLOT(songRequested(const QModelIndex&)));
        connect (tearOff, SIGNAL(tearOff()), this, SLOT(onTearOff()));
    }

    EnqueueList::~EnqueueList ()
    {
    }

    void EnqueueList::setTearOff (bool show)
    {
        if (show) {
            tearOff->show();
        }
        else tearOff->hide();
    }

    void EnqueueList::onTearOff()
    {
        emit openPlayDock(true);
        emit close();
    }

    void EnqueueList::removeSelection ()
    {
        QItemSelectionModel* sel = listView->selectionModel();
        if (sel->hasSelection()) {
            QModelIndexList items=sel->selection().indexes();
            foreach (QModelIndex i, items) {
                dynamic_cast<EnqueueModel*>(listView->model())->removeRow(i.row());
            }
        }
    }

}
}

#include "enqueuelist.moc"
