/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PLAYER_ENQUEUEMODEL_H
#define PLAYER_ENQUEUEMODEL_H

#include <QAbstractListModel>

#include "model/tagitem.h"

namespace Texel {

    namespace Player {

    class EnqueueModel: public QAbstractListModel
    {
        Q_OBJECT

    public:
        EnqueueModel (QObject* parent);
        virtual ~EnqueueModel();

        TagItem* getRowByFilename (const QString& name) const;

        void append (TagItem* item);
        void prepend(TagItem* item);

        bool isEmpty () const
        {
            return m_Queue.isEmpty();
        }

        virtual int rowCount (const QModelIndex & parent = QModelIndex()) const;
        virtual QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
        virtual Qt::ItemFlags flags ( const QModelIndex & index ) const;

        virtual bool dropMimeData (const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent);
        virtual QMimeData* mimeData (const QModelIndexList& indexes) const;
        virtual QStringList mimeTypes () const;
        virtual Qt::DropActions supportedDropActions() const;

        void removeRow (int row);

        int getPrev () const;
        int getNext () const;
        QString setSong (int i);

    signals:
        void songRequested (int song, bool enforce);

    public slots:
        void clear();
        void songRequested (const QModelIndex& index);

    private slots:
        void createPlaylist ();

    private:
        int m_PlayedIndex;
        TagItem::List m_Queue;
    };

}
}

#endif
