/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PLAYER_PLAYER_H
#define PLAYER_PLAYER_H

#include <QObject>

#include <Phonon/MediaObject>

#include <ktoolbar.h>

#include "enqueuemodel.h"
#include "util/singleton.h"

namespace Phonon {
    class MediaObject;
    class MediaSource;
    class AudioOutput;
}
class KStatusBar;
class QAbstractItemView;
class QMainWindow;
class QMenu;
class QToolButton;
class KNotification;

namespace Texel {
    class SlowClickToolButton;
    class ClickableLabel;

    namespace Player {

    class Player:public QObject, public Singleton<Player>
    {
        Q_OBJECT

        friend class Singleton<Player>;

    public:
        void setItemView (QAbstractItemView* itemview);

        void initToolBar (KToolBar* toolbar);
        void initCompleteToolBar (KToolBar* parent);

        void initStatusBar (KStatusBar* statusbar);
        QMenu* createMenu ();

        EnqueueModel* model();
        Phonon::MediaObject* mediaObject ();

    protected:

    private slots:
        void addAndPlaySong();
        void onPlayPressed ();

        void enqueueNextSong ();
        void playSong (int index, bool replace);
        void enqueueSong ();
        void displayElapsedTime (qint64 time);
        void toggleTimeDisplay();
        void songChanged (const Phonon::MediaSource &);

        void nextSong();
        void prevSong();
        void pause ();

        void notificationClosed();

        void onPhononStateChange (Phonon::State newstate, Phonon::State oldstate);

    signals:
        void error (const QString& msg);
        void playing (TagItem* item);
        void timeElapsed (const QString& time);
        void openPlayDock (bool);

    private:
        Player ();
        virtual ~Player();

        QToolButton*         m_EnqueueButton;
        SlowClickToolButton* m_PlayButton;
        SlowClickToolButton* m_StopButton;
        EnqueueModel         m_Queue;
        bool                 m_DisplayElapsedTime;
        ClickableLabel*      m_TimeLabel;
        Phonon::MediaObject* m_player;
        QAbstractItemView*   m_ItemView;
        Phonon::AudioOutput* out;

        QAction* actionPlay;
        QAction* actionStop;
        QAction* actionEnqueue;
        QAction* actionPause;
        QAction* actionClearList;
        QAction* actionAddAndPlay;
        QAction* actionPrev;
        QAction* actionNext;

        QMenu* m_Menu;

        KNotification* m_Notification;
    };

    inline EnqueueModel* Player::model ()
    {
        return &m_Queue;
    }

    inline Phonon::MediaObject* Player::mediaObject ()
    {
        return m_player;
    }

}
}

#endif
