/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef SEARCHWIDGET_H
#define SEARCHWIDGET_H

#include <QWidget>

#include "model/tagmodelproxy.h"

namespace Ui {
    class SearchWidget;
}

class QTableView;
class KLineEdit;

namespace Texel {

    class TagModelProxy;

    namespace Internal
    {
        class Comparator;
    }

    class SearchWidget : public QWidget
    {
        Q_OBJECT

    public:
        explicit SearchWidget(QWidget *parent = 0);
        ~SearchWidget();

        void setModel (TagModelProxy* model);
        void setTableWidget (QTableView* widget);

        virtual void setVisible (bool visible);

        void setup ();

    private slots:
        void searchForward ();
        void searchBackward ();
        void searchCurrent();
        void replaceForward ();
        void replaceBackward ();
        void replaceAll ();

        void searchPortion ();
        // When startAtCurrent is true the terator starts at the currently focused item,
        // otherwise a the beginning of the document
        void setStartIterator (const QModelIndex& start);
        void setComparator ();
        void resetComparator ();

    protected:
        void changeEvent(QEvent *e);

    private:
        void replace (const QModelIndex& idx, Internal::Comparator* comp);

        void addCurrentText (KLineEdit* editor);

        Ui::SearchWidget *ui;
        TagModelProxy* m_Model;
        QTableView* m_TableWidget;

        struct {
            TagModelProxy::iterator start;
            TagModelProxy::iterator searchPosition;
            Internal::Comparator* comp;
            bool forward;
        } m_Current;

        QAction* m_ActionWholeWords;
        QAction* m_ActionSelection;
        QAction* m_ActionRegExp;
        QAction* m_ActionCaseSensitive;
    };

    inline void SearchWidget::setModel (TagModelProxy* model)
    {
        m_Model=model;
    }

    inline void SearchWidget::setTableWidget (QTableView* widget)
    {
        m_TableWidget=widget;
    }

}

#endif // SEARCHWIDGET_H
