/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TEXTFILTERMODEL_H
#define TEXTFILTERMODEL_H

#include <QAbstractTableModel>
#include <QModelIndex>
#include <QList>

namespace Texel {

struct CompiledFinder
{
    virtual ~CompiledFinder ();
    virtual QString replace (const QString& source) const=0;
    virtual void insertFragment (const QString& pattern, const QString& replacement, bool beginOnly, bool word)=0;
};

class TextFilterModel : public QAbstractTableModel
{
   Q_OBJECT;

public:
    TextFilterModel(QObject* parent);

    virtual int columnCount (const QModelIndex & parent = QModelIndex()) const;
    virtual int rowCount (const QModelIndex & parent = QModelIndex()) const;

    virtual QVariant data (const QModelIndex& index, int role = Qt::DisplayRole ) const;
    virtual bool setData (const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
    virtual QVariant headerData (int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;

    virtual bool insertRows ( int row, int column, const QModelIndex & parent = QModelIndex() );
    virtual bool removeRows ( int row, int count, const QModelIndex & parent = QModelIndex());

    virtual bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex& parent);

    virtual QStringList mimeTypes () const;

    CompiledFinder* compile () const;

    bool isDirty () const;

public slots:
    void load (const QString& filename);
    void save (const QString& filename);

signals:
   void modelChanged();

protected:
    virtual Qt::DropActions supportedDropActions () const;
    virtual Qt::ItemFlags flags (const QModelIndex & index ) const;

private:
    struct Item
    {
       QString pattern;
       QString replacement;
       bool regex;
       bool beginOnly;
       bool wholeWord;

       Item& operator= (const Item& other)
       {
           pattern=other.pattern;
           replacement=other.replacement;
           regex=other.regex;
           beginOnly=other.beginOnly;
           wholeWord=other.wholeWord;
           return *this;
       }
    };

    bool m_ModelChanged;
    QList<Item> items;
};

inline bool TextFilterModel::isDirty () const
{
    return m_ModelChanged;
}

}

#endif // TEXTFILTERMODEL_H
