/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "textfiltersingleton.h"
#include "options/preferences.h"

namespace Texel {

    TextFilterSingleton::TextFilterSingleton()
        : model(new TextFilterModel(0l)),
        finder(0l),
        notifier(new Private::Notifier())
    {
        model->load(Preferences::instance()->getReadNamePatternsFile());
        notifier->connect (model, SIGNAL(modelChanged()), notifier, SLOT(setRecompile()));
    }

    TextFilterSingleton::~TextFilterSingleton()
    {
        delete model;
        delete finder;
        delete notifier;
    }

    TextFilterModel* TextFilterSingleton::textFilter ()
    {
        return model;
    }

    QString TextFilterSingleton::filter (const QString& s)
    {
        if (notifier->recompile()) {
            delete finder;
            finder=model->compile();
        }
        return finder->replace(s);
    }

    Private::Notifier::Notifier ()
        : m_Recompile(true)
    {
    }

    Private::Notifier::~Notifier()
    {
    }

    void Private::Notifier::setRecompile ()
    {
        m_Recompile=true;
    }

}

#include "textfiltersingleton.moc"
