/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TEXTFILTERSINGLETON_H
#define TEXTFILTERSINGLETON_H

#include <util/singleton.h>
#include <func/textfiltermodel.h>

namespace Texel
{

    namespace Private {
        /** As the singleton can't receive a signal this helper object (which can be derived from
          QObject) is introduced which can receive signals.
          Singletons can't be QObjects as the singleton class itself can is not a QObject; this
          leads to link problems.
          */
        class Notifier: public QObject
        {
            Q_OBJECT
            bool m_Recompile;

        public:
            Notifier ();
            virtual ~Notifier();

            /** Returns true, when a recompile signal was received since last request.
              */
            bool recompile()
            {
                if (m_Recompile) {
                    m_Recompile=false;
                    return true;
                }
                return false;
            }

        public slots:
            void setRecompile();
        };
    }

class TextFilterSingleton : public Singleton<TextFilterSingleton>
{
    TextFilterSingleton();
    virtual ~TextFilterSingleton();

    TextFilterModel* model;
    CompiledFinder* finder;
    Private::Notifier* notifier;

    friend class Singleton<TextFilterSingleton>;

protected:

public:
    TextFilterModel* textFilter ();

    void setRecompile ();
    QString filter (const QString& s);
};

}

#endif // TEXTFILTERSINGLETON_H
