/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <ktip.h>
#include <kstandarddirs.h>
#include <kicon.h>
#include <kdebug.h>

#include "texel.h"
#include "texelversion.h"
#include "func/coversearch/toolbox.h"
#include "func/player/player.h"
#include "func/coversearch/cover.h"
#include "global_exceptions.h"

#include "model/dirtree.h"
#include "model/tagitem.h"
#include "func/m3u/m3ulist.h"

namespace Texel {

    void doRegister ()
    {
        qRegisterMetaType<DirTree::AdditionalFileFormats>("DirTree::AdditionalFileFormats");
        qRegisterMetaType<TagItem::List>("TagItem::List");
        qRegisterMetaType<M3U::List>("M3U::List");
        qRegisterMetaType<QPair<QString,QString> >("QPair<QString,QString>");


    }

}

int main(int argc, char** argv)
{
    // Commandline handling
    KAboutData aboutData("texel","texel",ki18n("Texel"),TEXEL_VERSION);

    aboutData.setHomepage("http://texel.sf.net");
    aboutData.setShortDescription(ki18n("Texel allows to meta tag music files in an efficient way. Its look appears like "
                                              "a spread sheet with similar 'fill down' or formula capabilities. Still it allows "
                                              "to change tags even quickly when you are new to texel and have no experiences "
                                              "with writing or maintaining formulars."));
    aboutData.addAuthor(ki18n("Jens Ketterer"),ki18n("all"),"mb.jk@t-online.de");
    aboutData.setBugAddress("mb.jk@t.online.de");
    aboutData.setTranslator(ki18nc("NAME OF TRANSLATORS", "Your names"),
                            ki18nc("EMAIL OF TRANSLATORS", "Your emails"));
    aboutData.addLicense(KAboutData::License_GPL_V3);
    aboutData.setCopyrightStatement(ki18n("(c) 2009-2013"));

    KCmdLineOptions opts;
    opts.add ("+files",ki18n("optional directory or file to add to the tagger list"));

    KCmdLineArgs::addCmdLineOptions(opts);
    KCmdLineArgs::init(argc, argv, &aboutData);

    Texel::doRegister();

    try {
        KApplication app;
        app.setWindowIcon(KIcon(KIconLoader::global()->loadIcon("texel",KIconLoader::Small)));
        aboutData.setProgramLogo(KIconLoader::global()->loadIcon("texel",KIconLoader::Panel,KIconLoader::SizeLarge).toImage());

        // Create KDebug Instance early enough. Otherwise preferences is created to early and destruction fails
        kDebug ();

        // Restore old state
        Texel::texel* t=new Texel::texel();
        t->setAboutData(&aboutData);
        t->show();

        KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
        for (int i=0;i<args->count();i++) {
           t->addDir(KUrl::fromPath(args->arg(i)));
        }

        KTipDialog::showTip(t,KStandardDirs::locate("appdata", "tips"));

        t->checkEmptyModel();
        const int result=KApplication::exec();

        // limit lifetime explicitly as otherwise phonon singletons will crash
        Texel::Player::Player::destroy();

        return result;
    }
    catch (application_initialization_termination err) {
        std::cerr << i18n("SEVERE ERROR\nCoulnd't start application Texel because initalization went wrong: %1",err.what())
                  << std::endl;
        return err.getExitCode();
    }
}
