/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef texel_COLMNMANAGER_H
#define texel_COLMNMANAGER_H

#include <QVector>
#include <QMetaType>
#include <QRegExp>
#include <QHash>
#include <QMap>

#include <klocalizedstring.h>

#include <memory>

#include "util/singleton.h"
#include "aid.h"

class QFile;

namespace Texel {

class ColumnManager: public Singleton<ColumnManager>
{
public:
   enum ColumnNames { Pattern=0, DestinationPath, Path, Filename, Artist, Title, Album, 
                      Comment, Track, CD, Rating, Year, Genre, Length, Bitrate, Channels, SampleRate,
                      FileSize, MarkForDeletion,

                      Invalid, FirstUser };

   int getEditableColumns () const
   {
       return m_EditableColumns.size();
   }

   int mapToEditableColumn (int column) const;
   int mapFromEditableColumn (int editableColumn) const;

   int getColumnNumber () const
   {
      return m_Columns.size();
   }

   bool isEditable (int column) const
   {
      return m_Columns[column]->editType!=QMetaType::Void;
   }

   bool isSearchable (int column) const
   {
      return m_Columns[column]->searchable;
   }

   QString getName (int column) const
   {
      return m_Columns[column]->name.toString();
   }

   QMetaType::Type getType (int column) const
   {
      return m_Columns[column]->editType;
   }

   QMetaType::Type getSortType (int column) const
   {
      return m_Columns[column]->sortType;
   }

   const QString& getXmlName (int column) const
   {
       return m_Columns[column]->xmlName;
   }

   int indexByName (const QString& name) const;
   int indexByXmlName (const QString& name) const;

   ColumnNames getLeastPriority (int n) const;

   bool isHidden (int column) const
   {
      return m_Columns[column]->hidden;
   }

   bool streamToList (int column) const
   {
      return m_Columns[column]->streamToList;
   }

   QString getDescription (int col) const;

private:
   ColumnManager ();
   virtual ~ColumnManager ();

   void initFromXML (QFile* file,const QString& selection);

   struct Data
   {
      Data ();

      KLocalizedString name;
      QString commonName;
      QString xmlName;
      bool    searchable;
      bool    hidden;
      QMetaType::Type editType;
      QMetaType::Type sortType;
      bool    streamToList;
      KLocalizedString description;
   };

   void add (ColumnNames pos, Data* data);
   void calcEditables ();

   static QString asXml (const QString& name);

   QVector<std::auto_ptr<Data> > m_Columns;
   QList<ColumnNames> m_LeastPrios;
   QList<int> m_EditableColumns;
   QHash<QString,int> m_IndexXmlMapper;

   friend class Singleton<ColumnManager>;
};

}

#endif
