/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef COLUMNSBASE_H
#define COLUMNSBASE_H

#include "model/dependencymap.h"

class QCompleter;

namespace Texel {

class TagItem;
class TagModel;

namespace Columns {

///////////////////////////////////////////////////////////////////////////////////////////////
//
// Base class for all columns
//
//////////////////////////////////////////////////////////////////////////////////////////////

class Column
{
public:
    enum Role
    {
        /// Used to intalize the original value.
        RoleInitialization,
        /// Used to display the current value in the grid
        RoleDisplay,
        /// Used to gather the data for feeding the editor widget
        RoleForEdit,
        /// Used to retrieve the current value as variable
        RoleAsVariable,
        /// Used to retrieve the original value as variable
        RoleAsRawVariable,
        /// Used to retrieve the value for (XML) persistance
        RolePersistence,
        /// Used to retrieve the formular which is dedicated to this value, or on
        /// write when the result ofa formular is written (internally used)
        RoleFormular,
        /// Used when the data is reevaluated
        RoleReevaluate,
        /// Used when an update value is (re)nitalized
        RoleReInitalization,
        /// Used for Sorting: String is simplified
        RoleSorting,
        /// Used to displayformulars istead of the computed values, but keep otherwise the RoleDisplay
        RoleDisplayFormularFirst
    };

    Column ();
    virtual ~Column ();

    virtual QString data (Role role) const=0;
    virtual bool setData(const QString& value, Role role, TagItem* item)=0;

    virtual QString name () const=0;
    virtual int column() const=0;
    virtual bool isEditable () const=0;

    virtual QCompleter* getCompleter () const;

protected:
    DependencyMap& getDependencyMap(TagItem* item);
    TagModel* model(TagItem* item);
};

}

}

#endif // COLUMNSBASE_H
