/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "completermodel.h"
#include "aid.h"

#include <QtAlgorithms>

using namespace std;

namespace Texel {

CompleterModelIF::CompleterModelIF ()
: m_ActiveSet(0)
{
}

CompleterModelIF::~CompleterModelIF ()
{
}

CompleterModel::CompleterModel (QObject* parent)
: QAbstractListModel(parent),
  m_Data(2),
  m_Sorted(2)
{
    m_Sorted[0]=m_Sorted[1]=false;
}

CompleterModel::~CompleterModel ()
{
}

bool CompleterModel::setData (const QModelIndex&, const QVariant& , int)
{
   return false;
}

QVariant CompleterModel::data (const QModelIndex& index, int) const
{
    if (!index.isValid()) return QVariant();

    if (!m_Sorted[m_ActiveSet]) {
        qStableSort(m_Data[m_ActiveSet]);
        m_Sorted[m_ActiveSet]=true;
    }

    return QVariant(m_Data[m_ActiveSet][index.row()]);
}

int CompleterModel::rowCount (const QModelIndex& parent) const
{
    if (parent.isValid()) return 0;
    return m_Data[m_ActiveSet].size();
}

int CompleterModel::columnCount (const QModelIndex&) const
{
    return 1;
}

void CompleterModel::setData (int set, const QStringList& rows)

{
    if (m_Data.size()<=set) {
        m_Data.resize(set);
    }

    m_Data[set]=rows;
    reset();
}

void CompleterModel::insert (const QString& name)
{
    m_Data[m_ActiveSet].append(name);
    m_Sorted[m_ActiveSet]=false;
}

void CompleterModel::remove (const QString& name)
{
    QStringList::iterator iter = qBinaryFind(m_Data[m_ActiveSet].begin(),m_Data[m_ActiveSet].end(), name);
    if (iter!=m_Data[m_ActiveSet].end()) {
        m_Data[m_ActiveSet].erase(iter);
    }
}

void CompleterModel::setActiveSet (int set)
{
    const int old=m_ActiveSet;
    m_ActiveSet=set;
    if (old!=set) reset();
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
// QDirModel extension
//
////////////////////////////////////////////////////////////////////////////////////////////////

DirCompleterModel::DirCompleterModel (QObject* parent)
: QDirModel(parent)
{
}

DirCompleterModel::~DirCompleterModel()
{
}

QVariant DirCompleterModel::data (const QModelIndex& index, int role) const
{
    if (m_ActiveSet==0) return QDirModel::data(index,role);
    return m_Data[index.row()];
}

int DirCompleterModel::rowCount (const QModelIndex& parent) const
{
    if (m_ActiveSet==0) return QDirModel::rowCount(parent);
    return m_Data.size();
}

int DirCompleterModel::columnCount (const QModelIndex& parent) const
{
    if (m_ActiveSet==0) return QDirModel::columnCount(parent);
    return 1;
}

void DirCompleterModel::setActiveSet (int set)
{
    const int old = m_ActiveSet;
    m_ActiveSet=set;
    if (set!=old) reset();
}

void DirCompleterModel::setData (int set, const QStringList& rows)
{
    if (set==1) {
        m_Data=rows;
        if (m_ActiveSet==1) reset();
    }
}

bool DirCompleterModel::setData (const QModelIndex& index, const QVariant& value, int role)
{
    return QDirModel::setData(index,value,role);
}

}

#include "completermodel.moc"
