/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef COMPLETERMODEL_H
#define COMPLETERMODEL_H

#include <QAbstractListModel>
#include <QVector>
#include <QString>
#include <QStringList>
#include <QDirModel>

namespace Texel {

class CompleterModelIF
{
public:
   int getActiveSet () const;
   virtual void setActiveSet (int set)=0;
   virtual void setData (int set, const QStringList& rows)=0;

protected:
   CompleterModelIF();
   virtual ~CompleterModelIF();

   int m_ActiveSet;
};

inline int CompleterModelIF::getActiveSet () const
{
    return m_ActiveSet;
}

class CompleterModel: public QAbstractListModel, public CompleterModelIF
{
Q_OBJECT
public:
   CompleterModel (QObject* parent=NULL);
   virtual ~CompleterModel();

   virtual bool setData (const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
   virtual QVariant data (const QModelIndex & index, int role = Qt::DisplayRole ) const;
   virtual int rowCount (const QModelIndex & parent = QModelIndex() ) const;
   virtual int columnCount (const QModelIndex & parent = QModelIndex() ) const;

   virtual void setActiveSet (int set);
   virtual void setData (int set, const QStringList& rows);

   void insert (const QString& name);
   void remove (const QString& name);

private:
   mutable QVector<QStringList> m_Data;
   mutable QVector<bool> m_Sorted;
};

class DirCompleterModel: public QDirModel, public CompleterModelIF
{
Q_OBJECT
public:
   DirCompleterModel (QObject* parent=NULL);
   virtual ~DirCompleterModel();

   virtual QVariant data (const QModelIndex & index, int role = Qt::DisplayRole ) const;
   virtual int rowCount (const QModelIndex & parent = QModelIndex() ) const;
   virtual int columnCount (const QModelIndex & parent = QModelIndex() ) const;
   virtual bool setData (const QModelIndex& , const QVariant& , int);

   virtual void setActiveSet (int set);
   virtual void setData (int set, const QStringList& rows);

private:
   QStringList m_Data;
};

}

#endif // COMPLETERMODEL_H
