/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef DEPENDENCYMAP_H
#define DEPENDENCYMAP_H

#include <QBitArray>

#include "model/columnmanager.h"

class DependencyMap: private QBitArray
{
public:
    DependencyMap ()
        : QBitArray(size()*size())
    {
    }

    virtual ~DependencyMap()
    {
    }

    static int size ()
    {
        return Texel::ColumnManager::instance()->getEditableColumns();
    }

    void clear ()
    {
        QBitArray::fill(false);
    }

    bool testDependency (int from, int to) const
    {
        return testBit(index(from,to));
    }

    void clearDependency (int col)
    {
        fill (false, index(col,0), index(col,size()));
    }

    bool setDependency (int from, int to)
    {
        setBit(index(from,to),true);
        return checkCycle (from,to);
    }

    void clearDependency (int from, int to)
    {
        setBit(index(from,to),false);
    }

    QList<int> getDependencies (int col)
    {
        QList<int> result;
        getDeps (col,result);
        return result;
    }

    bool checkCycle (int from, int to)
    {
        return _checkCycle (to, from);
    }

private:
    static int index (int from, int to)
    {
        return from*size()+to;
    }

    void getDeps (int from, QList<int>& result)
    {
        for (int i=0;i<size();i++) {
            if (testDependency(from,i)) {
                result.append(i);
                getDeps(i,result);
            }
        }
    }

    bool _checkCycle (int from, int dest)
    {
        for (int i=0;i<size();i++) {
            if (testDependency(from,i)) {
                if (i==dest || _checkCycle(i,dest)) return true;
            }
        }
        return false;
    }
};

#endif // DEPENDENCYMAP_H
