/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef JOBTRACKER_H
#define JOBTRACKER_H

#include <QMap>

#include <kabstractwidgetjobtracker.h>

class QProgressBar;
class QPushButton;
class QEvent;
class QLabel;

namespace Texel {

    class JobTracker : public KAbstractWidgetJobTracker
    {
        Q_OBJECT

    public:
        JobTracker();
        virtual ~JobTracker();

        virtual bool eventFilter (QObject* watched, QEvent* event);

    public slots:
        virtual void registerJob (KJob *job);
        virtual void unregisterJob (KJob *job);

        virtual QWidget* widget(KJob *job);

    protected slots:
        virtual void description (KJob *job, const QString &title, const QPair< QString, QString > &field1, const QPair< QString, QString > &field2);
        virtual void percent (KJob *job, unsigned long percent);

    private slots:
        void closeTooltip ();

    signals:
        void percentExecuted (unsigned percent);
        void done ();

    private:
        struct JobInfo
        {
            JobInfo ();

            QString desc;
            int percent;
            bool percentageUnset;
        };

        QMap<KJob*, JobInfo> m_ControlledJobs;

        void createTrackerWidget ();
        void setTooltipText ();

        QWidget* m_Tracker;
        QProgressBar* m_ProgressBar;
        QPushButton* m_KillButton;

        QLabel* m_Tooltip;
        int m_UnsetJobs;
    };

}

#endif // JOBTRACKER_H
