/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef MODELLOADER_H
#define MODELLOADER_H

#include "model/tagitem.h"
#include "view/messageview.h"

#include <kjob.h>

namespace Texel {

    class TagModel;

    class ModelLoader: public KJob
    {
        Q_OBJECT
    public:
        ModelLoader (TagModel* parent);
        virtual ~ModelLoader();

        /** Adds a list of filename or directories to the loader. Can just be called when the loader
         is not running.
         @returns true, when setting succeded, false otherwise (loader is working)
         */
        bool addFileNames (const QList<QUrl>& filelist);

        virtual void start ();

    signals:
        void loadedItems (const TagItem::List items);
        void loadPlaylist (const QString& playlist);
        void loadAdditionalFile(const QString& file, DirTree::AdditionalFileFormats format);

    protected:
        virtual bool doKill ();

    private slots:
        void onFinished ();

    private:
        class Private;
        Private* m_Thread;
        friend class Private;
    };

}

#endif // MODELLOADER_H
