/*************************************************************************
*************************************************************************/

#ifndef texel_TAGMODELXMLSTREAMER_H
#define texel_TAGMODELXMLSTREAMER_H

#include <model/tagitem.h>

#include <QString>
#include <QThread>

#include <kjob.h>

namespace Texel {
    class TagModel;
    class ArchiveSave;
    class ArchiveLoad;

    namespace Internal {
        class ArchiveSaveWorker: public QThread
        {
            Q_OBJECT
        public:
            class Private;
            Private* p;

            ArchiveSaveWorker (TagModel& model, const QString& filename, ArchiveSave* job);
            virtual ~ArchiveSaveWorker ();

            virtual void run ();
        };

        class ArchiveLoadWorker: public QThread
        {
            Q_OBJECT
        public:
            class Private;
            Private* p;

            ArchiveLoadWorker (TagModel& model, const QString& filename, ArchiveLoad* job);
            virtual ~ArchiveLoadWorker ();

            virtual void run ();
        };
    }

    class TagListArchive: public KJob
    {
        Q_OBJECT

    public:
        TagListArchive (QThread *worker);
        virtual ~TagListArchive ();

        void setRows(int rows);
        void incRows();

    protected:
        virtual bool doKill ();

        QThread* m_Thread;
        volatile bool m_Stopped;

    protected slots:
        virtual void onFinished ();

    private:
        int m_ProcessedRows;
        int m_NumRows;
    };

    class ArchiveSave: public TagListArchive
    {
        Q_OBJECT

    public:
        ArchiveSave (const QString& filename, TagModel& model);
        virtual ~ArchiveSave ();
        virtual void start ();

    protected slots:
        virtual void onFinished ();

    signals:
        void errorOccured (const QString& archive, const QString& message);
        void saveListStatisticHint (const QString& filename, bool saved, int delta);

    private:
        TagModel& m_Model;
        friend class Internal::ArchiveSaveWorker::Private;
    };

    class ArchiveLoad: public TagListArchive
    {
        Q_OBJECT
    public:
        ArchiveLoad (const QString& filename, Texel::TagModel& model);
        virtual ~ArchiveLoad();
        virtual void start ();

    signals:
        void errorOccured (const QString& archive, const QString& what);
        void newTags (TagItem::List items);
        void newPlaylist (M3U::List list);
        void saveListStatisticHint (const QString& filename, bool saved, int delta);
        void addDuplicateRelation (int row, const QList<int>& dups);

    private:
        Texel::TagModel& m_Model;
        friend class Internal::ArchiveLoadWorker::Private;
    };

}

#endif
