/*************************************************************************
    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
*************************************************************************/

#include "problem.h"
#include "model/tagitem.h"
#include "util/widgets/tooltip.h"

#include <QPalette>
#include <QLabel>
#include <QWidget>
#include <QTimerEvent>
#include <QCursor>
#include <QAbstractTableModel>
#include <QGridLayout>
#include <QToolButton>

#include "aid.h"

using namespace std;

namespace Texel {

    InputProblem::InputProblem() :
            QObject(0l)
    {
    }

    InputProblem::~InputProblem()
    {
    }

    void InputProblem::showTooltip(const QPoint& pos, const QRect& tooltipBox)
    {
        QWidget* tooltip = new QWidget(0l, Qt::ToolTip);
        QPalette p = tooltip->palette();
        p.setColor(QPalette::Background, QColor(240,100,100));
        tooltip->setPalette(p);
        QLabel* label = new QLabel(getText()+"\n"+locator(), tooltip);
        QFont font=label->font();
        font.setPointSize(10);
        label->setFont(font);
        label->setWordWrap(true);
        label->setFrameShape(QFrame::Box);

        QList<QAction*> actions = getActions();
        QGridLayout* layout=new QGridLayout(tooltip);
        layout->addWidget(label, 0,0,1,actions.size());
        for (int i=0;i<actions.size();i++) {
            QToolButton* b=new QToolButton(tooltip);
            b->setDefaultAction(actions[i]);
            layout->addWidget(b,1,i);
        }
        tooltip->setLayout(layout);

        Tooltip::show(tooltip,pos,tooltipBox, 0l);
    }

    QList<QAction*> InputProblem::getActions ()
    {
        return QList<QAction*>();
    }

    void InputProblem::setIndex (const QModelIndex& _index)
    {
        index=_index;
    }

    bool InputProblem::isFormularProblem() const
    {
        return false;
    }

    void InputProblem::setData (const QString& data)
    {
        // Create a local copy as setData takes a reference to the index and passes it, but
        // while execution the current instance is deleted. Otherwise a stale reference would exist
        const QModelIndex localCopy=index;
        const_cast<QAbstractItemModel*>(localCopy.model())->setData(localCopy,data);
    }

    QString InputProblem::data () const
    {
        return index.data().toString();
    }

    UnknownProblem::UnknownProblem (const QString& desc)
        : description(desc)
    {
    }

    UnknownProblem::~UnknownProblem ()
    {
    }

    QString UnknownProblem::getText () const
    {
        return description;
    }

    void UnknownProblem::setLocator (const QString locator)
    {
        m_Locator=locator;
    }

    QString UnknownProblem::locator() const
    {
        return m_Locator;
    }

}
#include "problem.moc"
