/*************************************************************************
    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
*************************************************************************/

#ifndef PROBLEM_H
#define PROBLEM_H

#include <QObject>
#include <QString>
#include <QPoint>
#include <QList>
#include <QAction>
#include <QModelIndex>

class QWidget;

namespace Texel {
    
    class InputProblem: public QObject
    {
        Q_OBJECT
        
    public:
        InputProblem();
        virtual ~InputProblem();
        
        virtual QString getText () const=0;
        virtual void showTooltip (const QPoint& pos, const QRect& tooltipBox);
        virtual QList<QAction*> getActions ();
        
        virtual void setLocator (const QString locator)=0;
        virtual QString locator() const=0;
        virtual bool isFormularProblem() const;
        
        void setIndex (const QModelIndex& index);
        
    protected:
        virtual void setData (const QString& data);
        virtual QString data () const;
        
    private:
        QModelIndex index;
    };

    class UnknownProblem: public InputProblem
    {
    public:
        UnknownProblem (const QString& desc);
        virtual ~UnknownProblem ();
        virtual QString getText () const;

        virtual void setLocator (const QString locator);
        virtual QString locator() const;

    private:
        QString description;
        QString m_Locator;
    };    

}

#endif // PROBLEM_H
