/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TEXEL_ARTISTMANAGER_H
#define TEXEL_ARTISTMANAGER_H

#include <QString>
#include <QStringList>
#include <QMap>
#include <QSet>
#include <QVector>
#include <QMetaType>

namespace Texel {

class CompleterModel;
class ManagedObject;

class ArtistManager: public QObject
{
    Q_OBJECT

public:
    enum State { ResultAvailable=(1<<1),
                 IsCorrect=(1<<2),
                 HasProposals=(1<<3),
                 MinorSpellErrors=(1<<4)
               };

    class Artist
    {
    public:
        Artist (const QString& name="");

        ~Artist ()
        {
        }

        const QString& getName () const
        {
            return name;
        }

        bool isCorrect () const
        {
            return (state&IsCorrect)!=0;
        }

        bool hasProposals () const
        {
            return state&HasProposals;
        }

        unsigned getState () const
        {
            return state;
        }

        void setState (unsigned state_)
        {
            const bool changed=state!=state_;
            state=state_;
            if (changed) {
                updateArtists();
            }
        }

        void add (ManagedObject* obj)
        {
            if (!items.contains(obj)) {
                items.append(obj);
            }
            obj->updateArtist(state);
        }

        const QStringList& getProposals () const
        {
            return proposals;
        }

        void remove (ManagedObject* obj);

    private:
        void updateArtists ()
        {
            foreach (ManagedObject* obj, items) {
                obj->updateArtist(state);
            }
        }

        QString name;
        unsigned state;
        QStringList proposals;
        QVector<ManagedObject*> items;

        friend class ArtistManager;
    };

    ArtistManager ();
    virtual ~ArtistManager();

    Artist* storeName (const QString& name, ManagedObject* );
    void changeName (const QString& oldName, const QString& newName);
    void deleteName (const QString& name);
    void removeObject (const QString& name, ManagedObject* obj);

    void reemitAll ();
    void resetAll ();

    CompleterModel* getModel () const;

signals:
    void newArtist (const QVariant& ref, const QString& name);

private slots:
    void newArtistInformation (const QVariant& ref, QStringList proposals);

private:
    QMap<QString, Artist*> storage;
    CompleterModel* model;
};

inline CompleterModel* ArtistManager::getModel () const
{
    return model;
}

}

Q_DECLARE_METATYPE(Texel::ArtistManager::Artist*)

#endif
