/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TEXEL_MASTERMANAGER_H
#define TEXEL_MASTERMANAGER_H

#include <QString>
#include <QObject>
#include <QStringList>
#include <QPair>

#include "util/singleton.h"

namespace Texel {

namespace CheckService {
   class CheckArtist;
}

class ManagedObject
{
public:
    virtual ~ManagedObject();

    virtual void updateArtist (int state)=0;
    virtual void changeArtistName (const QString& name)=0;
};

class ArtistManager;

class MasterManager:public QObject, public Singleton<MasterManager>
{
    Q_OBJECT

    friend class Singleton<MasterManager>;
    MasterManager ();
    ~MasterManager ();

public:
    ArtistManager* getArtistManager() const
    {
        return m_ArtistManager;
    }

    void createBackgroundService ();

    QPair<QStringList,QStringList> getCacheInformation () const;

public slots:
    void installBackgroundService ();

private:
    ArtistManager* m_ArtistManager;
    CheckService::CheckArtist* m_BackgroundService;

    enum BackgroundService { NoBackgroundService, MusicBrainzBackgroundService, EchonestBackgroundService };
    BackgroundService m_BackgroundServiceState;
};

}

#endif
