/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "tagrelations.h"
#include "model/tagitem.h"
#include "model/texeltagmodel.h"

namespace Texel {

    TagRelations::TagRelations()
    {
    }

    void TagRelations::addDuplicate (TagItem* item)
    {
        m_Duplicates << item;
        if (m_Duplicates.size()==1) item->getModel()->incDuplicateCounter(1);
    }

    void TagRelations::onRemovalFromList (TagItem* item)
    {
        removeDuplicates(item);
    }

    void TagRelations::onRemovalFromDisc (TagItem* item)
    {
        removeDuplicates(item);
        // When the track is removed physically it should also be removed from all playlists
        m_Playlists.removeTrack(item);
    }

    // Remove 'this' item from all as duplicate marked items.
    void TagRelations::removeDuplicates (TagItem *item)
    {
        foreach (TagItem* other, getDuplicates()) {
            if (other->hasRelations()) {
                other->getRelations()->removeDuplicate(item);
            }
        }
        m_Duplicates.clear();
    }

    void TagRelations::addPlaylist (M3U::List playlist, TagItem* item)
    {
        const bool empty = m_Playlists.size()==0;
        m_Playlists.addPlaylist(playlist);
        if (empty) {
            item->getModel()->emitChangedHeader(item->getRow());
        }
    }

}
