/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TAGRELATIONS_H
#define TAGRELATIONS_H

#include <QList>

#include <func/m3u/m3ulist.h>
#include <func/m3u/playlistgroup.h>

namespace Texel {

    class TagItem;

    class TagRelations
    {
    public:
        TagRelations();

        int sizeOfDuplicates () const;
        QList<TagItem*> getDuplicates () const;
        void clearDuplicates ();
        void addDuplicate (TagItem* item);
        void removeDuplicate (TagItem* item);

        void addPlaylist (M3U::List playlist, TagItem* item);
        void removePlaylist (M3U::List playlist);
        int numberOfPlaylists () const;
        QList<M3U::List> getPlaylists () const;
        PlaylistGroup& getPlaylistGroup ();

        void onRemovalFromList (TagItem* item);
        void onRemovalFromDisc (TagItem* item);

        void incPlayListDirtyCounter ();
        void decPlayListDirtyCounter ();

    private:
        void removeDuplicates (TagItem* item);

        QList<TagItem*> m_Duplicates;
        PlaylistGroup m_Playlists;
    };

    inline int TagRelations::sizeOfDuplicates () const
    {
        return m_Duplicates.size();
    }

    inline QList<TagItem*> TagRelations::getDuplicates () const
    {
        return m_Duplicates;
    }

    inline void TagRelations::clearDuplicates ()
    {
        m_Duplicates.clear();
    }

    inline void TagRelations::removeDuplicate (TagItem* item)
    {
        m_Duplicates.removeOne(item);
    }

    inline void TagRelations::removePlaylist (M3U::List playlist)
    {
        m_Playlists.removePlaylist(playlist);
    }

    inline int TagRelations::numberOfPlaylists () const
    {
        return m_Playlists.size();
    }

    inline QList<M3U::List> TagRelations::getPlaylists () const
    {
        return m_Playlists.getPlaylists();
    }

    inline void TagRelations::incPlayListDirtyCounter ()
    {
        m_Playlists.incDirtyCounter();
    }

    inline void TagRelations::decPlayListDirtyCounter ()
    {
        m_Playlists.decDirtyCounter();
    }

    inline PlaylistGroup& TagRelations::getPlaylistGroup ()
    {
        return m_Playlists;
    }

}

#endif // TAGRELATIONS_H
