/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TAGSPECIALIZATIONS_H
#define TAGSPECIALIZATIONS_H

#include "model/tagitem.h"
#include "model/tagextension.h"

#include "QByteArray"

#include <taglib/taglib.h>

namespace TagLib
{
    class Tag;
    class File;
    class FileRef;

    namespace MP4
    {
        class Tag;
    }

    namespace ID3v2
    {
        class Tag;
    }
}

namespace Texel {
    class ExtensionAwareItem: public TagItem
    {
    public:
        ExtensionAwareItem (TagModel* _model);
        virtual ~ExtensionAwareItem ();

        virtual bool isAltered () const;

        virtual TagExtension* getExtensions () const;
        virtual void resetExtension() const;

    protected:
        QByteArray convertImage (const QImage& image);
        mutable TagExtension* m_Ext;

    private:
        virtual TagExtension* createExtension (TagLib::FileRef& file) const=0;
    };

    class TagMPEG: public ExtensionAwareItem
    {
    public:
        virtual ~TagMPEG();
        TagMPEG (TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual void writeExtensions (TagExtension* ext);
        virtual bool supportsCoverImages() const;
        virtual QImage getDefaultCoverImage () const;
        virtual QString getDefaultImageName () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void doReadMetaTag (TagLib::FileRef& f, const QFileInfo& info);
        virtual void doSaveMetaTag (TagLib::File& f, TagLib::Tag* tag);
        virtual void saveMetaTag () throw (TagItem::IOError);
        void writeExtensions (TagExtension* ext, TagLib::ID3v2::Tag* tag);

    private:
        virtual TagExtension* createExtension (TagLib::FileRef& file) const;

        static QMap<int,QString> s_AllowedImagesMap;
        static QMap<QString,QString> s_AllowedAttributesMap;
        static QSet<QString> s_AllowedAttributesKey;
        static QSet<QString> s_AllowedImages;
        static TagExtension::AllowedAttributeMap s_AllowedAttributes;
    };

    class TagOggSpeex: public TagItem
    {
    public:
        virtual ~TagOggSpeex();
        TagOggSpeex(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagOggFLAC: public TagItem
    {
    public:
        virtual ~TagOggFLAC();
        TagOggFLAC(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagOggVorbis: public TagItem
    {
    public:
        virtual ~TagOggVorbis();
        TagOggVorbis(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagFLAC: public TagItem
    {
    public:
        virtual ~TagFLAC();
        TagFLAC(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagMPC: public TagItem
    {
    public:
        virtual ~TagMPC();
        TagMPC(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagTrueAudio: public TagItem
    {
    public:
        virtual ~TagTrueAudio();
        TagTrueAudio(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagWavPack: public TagItem
    {
    public:
        virtual ~TagWavPack();
        TagWavPack(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

#if (TAGLIB_MAJOR_VERSION>1) || ((TAGLIB_MAJOR_VERSION==1) && (TAGLIB_MINOR_VERSION>5))

    class TagMP4: public ExtensionAwareItem
    {
    public:
        virtual ~TagMP4();
        TagMP4(TagModel* _model);

        virtual void writeExtensions (TagExtension* ext);
#if (TAGLIB_PATCH_VERSION>0)
        virtual bool supportsCoverImages() const;
        virtual QImage getDefaultCoverImage () const;
        virtual QString getDefaultImageName () const;
#endif
        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void doReadMetaTag (TagLib::FileRef& f, const QFileInfo& info);
        virtual void doSaveMetaTag (TagLib::File& f, TagLib::Tag* tag);
        virtual void saveMetaTag () throw (TagItem::IOError);
        void writeExtensions (TagExtension* ext, TagLib::MP4::Tag* tag);

    private:
        virtual TagExtension* createExtension (TagLib::FileRef& file) const;

        static QSet<QString> s_AllowedImages;
        static TagExtension::AllowedAttributeMap s_AllowedAttributes;
        static QSet<QString> s_AllowedAttributesKey;
        static QMap<QString,QString> s_AllowedAttributesMap;

    };

    class TagASF: public TagItem
    {
    public:
        virtual ~TagASF();
        TagASF(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagRIFFAIFF: public TagItem
    {
    public:
        virtual ~TagRIFFAIFF();
        TagRIFFAIFF(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

    class TagRIFFWAV: public TagItem
    {
    public:
        virtual ~TagRIFFWAV();
        TagRIFFWAV(TagModel* _model);

        virtual QString typeName () const;
        virtual QString suggestFilenameExtension () const;
        virtual TagItemType::Type type () const;

    protected:
        virtual void saveMetaTag () throw (TagItem::IOError);
    };

#endif
}
#endif // TAGSPECIALIZATIONS_H
