/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "appearanceoption.h"
#include "ui_appearanceoption.h"

#include <QPainter>
#include <QWidget>
#include <QStandardItemModel>

AppearanceOption::AppearanceOption(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AppearanceOption),
    model(new QStandardItemModel(this))
{
    ui->setupUi(this);
    ui->examples->setModel(model);
    drawHeaderExample();
    drawNormalExample();

    connect (ui->kcfg_fontHeader, SIGNAL(fontSelected(QFont)), this, SLOT(drawHeaderExample()));
    connect (ui->kcfg_fontNormal, SIGNAL(fontSelected(QFont)), this, SLOT(drawNormalExample()));
    connect (ui->kcfg_colorHeader, SIGNAL(changed(QColor)), this, SLOT(drawHeaderExample()));
    connect (ui->kcfg_colorNormal, SIGNAL(changed(QColor)), this, SLOT(drawNormalExample()));
    connect (ui->kcfg_colorNormal, SIGNAL(changed(QColor)), this, SLOT(drawAlteredExample()));
    connect (ui->kcfg_fontNormal, SIGNAL(fontSelected(QFont)), this, SLOT(drawAlteredExample()));
    connect (ui->kcfg_colorAlteredBackground, SIGNAL(changed(QColor)), this, SLOT(drawAlteredExample()));
    connect (ui->kcfg_colorBackgroundHeader, SIGNAL(changed(QColor)), this, SLOT(drawHeaderExample()));
    connect (ui->kcfg_colorBackgroundNormal, SIGNAL(changed(QColor)), this, SLOT(drawNormalExample()));
}

AppearanceOption::~AppearanceOption()
{
    delete ui;
    delete model;
}

void AppearanceOption::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void AppearanceOption::drawHeaderExample()
{
    QStandardItem* item = new QStandardItem("Header");
    item->setFont(ui->kcfg_fontHeader->font());
    item->setBackground(ui->kcfg_colorBackgroundHeader->color());
    item->setForeground(ui->kcfg_colorHeader->color());
    model->setItem(0,0,item);
}

void AppearanceOption::drawNormalExample()
{
    QStandardItem* item = new QStandardItem("Normal");
    item->setFont(ui->kcfg_fontNormal->font());
    item->setBackground(ui->kcfg_colorBackgroundNormal->color());
    item->setForeground(ui->kcfg_colorNormal->color());
    model->setItem(1,0,item);
}

void AppearanceOption::drawAlteredExample ()
{
    QStandardItem* item = new QStandardItem("Altered");
    item->setFont(ui->kcfg_fontNormal->font());
    item->setBackground(ui->kcfg_colorAlteredBackground->color());
    item->setForeground(ui->kcfg_colorNormal->color());
    model->setItem(2,0,item);
}

#include "appearanceoption.moc"
