/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "coveroptions.h"
#include "ui_coveroptions.h"
#include "func/coversearch/toolbox.h"
#include "func/coversearch/coversearchwizard.h"
#include "options/preferences.h"

#include <QButtonGroup>
#include <QVBoxLayout>

namespace Texel {

    CoverOptions::CoverOptions(QWidget *parent) :
            QWidget(parent),
            ui(new Ui::CoverOptions)
    {
        ui->setupUi(this);
        ui->coverWizardButton->setIcon(KIconLoader::global()->loadIcon("tools-wizard",KIconLoader::Toolbar));

        QButtonGroup* group = new QButtonGroup(this);
        group->addButton(ui->kcfg_SaveAsJPG);
        group->addButton(ui->SaveAsPNG);
        group->setExclusive(true);

        KDialog* dialog = dynamic_cast<KDialog*>(parent);
        if (dialog!=0l) {
            connect (dialog, SIGNAL(applyClicked()), this, SLOT(onApply()));
            connect (dialog, SIGNAL(okClicked()), this, SLOT(onApply()));
            connect (this, SIGNAL(isDirty(bool)), dialog, SLOT(enableButtonApply(bool)));
        }

        ui->coverSourcesPlane->setLayout (new QVBoxLayout(ui->coverSourcesPlane));
        QList<CoverSearch::XQueryCoverSearcher*> searchers = CoverSearch::XQueryCoverSearcher::getConfiguredSearchers(false,this);
        foreach (CoverSearch::XQueryCoverSearcher* searcher, searchers) {
            QCheckBox* checkbox=new QCheckBox(searcher->title(),ui->coverSourcesPlane);
            checkbox->setChecked(Preferences::instance()->htmlgrabber.isCoverGrabberActive(searcher->title()));
            checkbox->setToolTip(searcher->description());
            checkbox->setProperty("source",searcher->title());
            ui->coverSourcesPlane->layout()->addWidget(checkbox);
            connect (checkbox, SIGNAL(toggled(bool)), this, SLOT(onSelectCoverSource(bool)));
        }

        connect (ui->coverWizardButton, SIGNAL(clicked()), this, SLOT(onOpenCoverWizard()));
    }

    CoverOptions::~CoverOptions()
    {
        delete ui;
    }

    void CoverOptions::onOpenCoverWizard ()
    {
        CoverSearch::CoverSearchWizard* wizard = new CoverSearch::CoverSearchWizard(this);
        wizard->show();
    }

    void CoverOptions::onApply ()
    {
        for (QMap<QString,bool>::iterator iter=m_ChangedActivationCoverSource.begin();
        iter!=m_ChangedActivationCoverSource.end();iter++) {
            Preferences::instance()->htmlgrabber.setGrabberState(iter.key(),iter.value());
        }
    }

    void CoverOptions::onSelectCoverSource (bool checked)
    {
        QObject* s = sender();
        QString source = s->property("source").toString();
        if (source!="") {
            m_ChangedActivationCoverSource.insert(source,checked);
            emit isDirty(true);
        }
    }

    void CoverOptions::changeEvent(QEvent *e)
    {
        QWidget::changeEvent(e);
        switch (e->type()) {
        case QEvent::LanguageChange:
            ui->retranslateUi(this);
            break;
        default:
            break;
        }
    }

}
