/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "externalresources.h"
#include "ui_externalresources.h"
#include "func/metatagservices/freedb/freedbaccess.h"
#include "func/metatagservices/freedb/indexdownloader.h"
#include "util/persistentcache.h"
#include "options/preferences.h"
#include "util/loghook.h"

#include <QCheckBox>

#include <kmessagebox.h>
#include <kprogressdialog.h>

ExternalResources::ExternalResources(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ExternalResources)
{
    ui->setupUi(this);

    connect (ui->kcfg_ENenabled, SIGNAL(toggled(bool)), this, SLOT(onToggleBackgroundService(bool)));
    connect (ui->downloadButton, SIGNAL(clicked()), this, SLOT(onDownloadClicked()));
    connect (ui->compressButton, SIGNAL(clicked()), this, SLOT(onCompressCache()));

#ifndef TEXEL_MUSICBRAINZ_SUPPORT
    ui->tabWidget->removeTab(ui->tabWidget->indexOf(ui->musicbrainz));
#else
    connect (ui->kcfg_MBenabled, SIGNAL(toggled(bool)), this, SLOT(onToggleBackgroundService(bool)));
#endif
}

ExternalResources::~ExternalResources()
{
    delete ui;
}

void ExternalResources::onCompressCache()
{
    ui->compressButton->setDisabled(true);
    KProgressDialog* progressDialog = new KProgressDialog(0,i18n("Compress cache"));
    progressDialog->setAllowCancel(false);
    progressDialog->show();

    try {
        Texel::PersistentCache::instance()->compress();
        Texel::LogHook::instance()->message (i18n("Compressed persistant cache"),0l);
    }
    catch (Texel::SqLite::Exception e) {
        Texel::LogHook::instance()->error (i18n("Error while compressing cache: %1",e.msg),0l);
        KMessageBox::error(0,i18n("Error while compressing cache: %1",e.msg),i18n("Compress cache"));
    }

    progressDialog->hide();
    progressDialog->deleteLater();
    ui->compressButton->setEnabled(true);
}

void ExternalResources::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void ExternalResources::onToggleBackgroundService (bool checked)
{
    if (checked) {
        QObject* s=sender();
        if (s==ui->kcfg_ENenabled) {
            ui->kcfg_MBenabled->setChecked(false);
        }
        if (s==ui->kcfg_MBenabled) {
            ui->kcfg_ENenabled->setChecked(false);
        }
    }
}

void ExternalResources::onDownloadClicked()
{
    if (Texel::FreeDB::FreeDBAccess::existIndex()) {
        int result=KMessageBox::questionYesNo (0l,
                i18n("Downloading a new index file will overwrite the existing file\n%1", Texel::Preferences::instance()->freedb.m_IndexFile),
                i18n("Download and overwrite index")
                );
        if (result==KMessageBox::No) {
            return;
        }
    }
    else {
        int result=KMessageBox::questionYesNo (0l,
                i18n("Download index to %1", Texel::Preferences::instance()->freedb.m_IndexFile),
                i18n("Download index")
                );
        if (result==KMessageBox::No) {
            return;
        }
    }

    new Texel::FreeDB::IndexDownloader(this);
}

#include "externalresources.moc"
