/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "filteroption.h"
#include "func/textfiltersingleton.h"
#include "options/preferences.h"

#include <QTableView>
#include <QStyledItemDelegate>
#include <QFileInfo>
#include <QPushButton>

#include <kcharselect.h>
#include <kconfigdialog.h>
#include <klocale.h>

namespace Texel
{

    FilterOption::FilterOption(KConfigDialog* parent)
        : TableEditor(parent),
        m_CharSelector()
    {
        init();

        table()->viewport()->setAcceptDrops(true);
        table()->setAcceptDrops(true);
        table()->setDropIndicatorShown(true);
        table()->setDragDropMode(QAbstractItemView::DropOnly);

        QPushButton* charSelectorButton=new QPushButton(i18n("Symbol ..."));
        addButton(charSelectorButton);
        connect (charSelectorButton, SIGNAL(clicked()), this, SLOT(selectSymbol()));

        connect (parent, SIGNAL(applyClicked()), this, SLOT(save()));
        connect (parent, SIGNAL(okClicked()), this, SLOT(save()));
        //connect (parent, SIGNAL(resetClicked()), this, SLOT(load()));
        connect (model(), SIGNAL(modelChanged()), parent, SLOT(settingsChangedSlot()));
    }

    TextFilterModel* FilterOption::model ()
    {
        return dynamic_cast<TextFilterModel*>(TableEditor::model());
    }

    QAbstractItemModel* FilterOption::doInitModel (QWidget*) const
    {
        return TextFilterSingleton::instance()->textFilter();
    }

    int FilterOption::doAppend ()
    {
        const int row=model()->rowCount();
        model()->insertRow (row+1);
        return row;
    }

    void FilterOption::selectSymbol ()
    {
        if (m_CharSelector==0l) {
            m_CharSelector = new KCharSelect(0l,0l,KCharSelect::CharacterTable|KCharSelect::BlockCombos);
            connect (m_CharSelector, SIGNAL(charSelected(QChar)), this, SLOT(selectedChar(QChar)));
        }
        m_CharSelector->show();
        m_CharSelector->raise();
    }

    void FilterOption::selectedChar(QChar c)
    {
        QString v = model()->data(table()->currentIndex()).toString();
        v+=c;
        model()->setData(table()->currentIndex(),v);
    }

    void FilterOption::load ()
    {
        QFileInfo info(Preferences::instance()->getReadNamePatternsFile());
        if (info.exists()) {
            model()->load(info.absoluteFilePath());
        }
    }

    void FilterOption::save ()
    {
        if (model()!=0l && model()->isDirty()) {
            model()->save(Preferences::instance()->getWriteNamePatternsFile());
        }
    }

}

#include "filteroption.moc"
