/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <KConfigSkeleton>

#include <QString>

#include "util/singleton.h"
#include "options/enums.h"

#include "kurl.h"

namespace Texel {

class Preferences : public KConfigSkeleton, public Singleton<Preferences>
{
    friend class Singleton<Preferences>;

    void readGlobalVars ();
    void writeGlobalVars ();

public:
    Preferences();
    ~Preferences ();

    static Preferences* self ();

    struct SearchOptions {
        bool m_Persist;
        int  m_Historysize;
        bool m_CaseSensitive;
        bool m_WholeWords;
        bool m_RegExp;
        bool m_Selection;
    } search;

    struct IOOptions {
        bool m_AckFileRemove;
        bool m_AckDirRemove;
        bool m_RemoveEmptyDir;
        bool m_OverwriteOnRename;
        bool m_UseTrashCan;
        QString m_TrashCan;
        bool m_OverwriteInTrashCan;
        bool m_TryEnforceWrite;
        QString m_TrackPatterns;
        QString m_PlaylistPatterns;
    } io;

    // Dulicate detection
    int m_DD_maxDuplicateDisplay;
    int m_DD_maxErrorInARow;

    // File locations
    struct FileLocations {
        QString m_ReplacePatternsFile;
        QString m_PhrasesFile;

        bool m_UseLastDirectory;
        QString m_LastAccessedDirectory;
        QString m_DefaultDirecory;
        QString m_DefaultPlaylistName;
    } filelocs;

    struct HtmlGrabber{
        bool isCoverGrabberActive (const QString& grabber) const;
        void setGrabberState (const QString& grabber, bool active);

        QStringList m_ActiveCoverGrabbers;
    } htmlgrabber;

    QString getReadNamePatternsFile () const;
    /**
      This function returns not only the configured file but does also ask the user when the local
      default is configured but the global file is writeable. Then the user can chose which one to
      overwrite.
      */
    QString getWriteNamePatternsFile () const;
    QString getReadNamePhrasesFile () const;
    /**
      This function returns not only the configured file but does also ask the user when the local
      default is configured but the global file is writeable. Then the user can chose which one to
      overwrite.
      */
    QString getWriteNamePhrasesFile () const;

    struct Debugging {
        bool m_TraceDragging;
    } debug;

    struct Appearance {
        QFont m_normalFont;
        QFont m_headerFont;
        QColor m_normalForeground;
        QColor m_normalBackground;
        QColor m_headerForeground;
        QColor m_headerBackground;
        QColor m_alteredBackground;
        QColor m_markedForDeletionForeground;

        bool m_ShowHelpLinks;
    } appearance;

    struct MusicBrainz {
        bool m_Enabled;
        QString m_URL;
    } musicbrainz;

    struct PersistentCache {
        QString m_CacheFile;
        int m_MaxFileSize;
        int m_OldestEntriesInDays;
    } cache;

    struct FreeDB {
        QString m_Url;
        double  m_Range;
        QString m_IndexFile;
        QString m_IndexDownloadURL;
    } freedb;

    struct EchoNest {
        bool m_Enabled;
        QString m_URL;
    } echonest;

    struct DuplicateHelperFilter {
        Enums::FilterProperties m_Properties;
        QStringList m_IncludeDirectories;
    } duplicatehelper;

    struct Behaviour {
        bool m_RestoreDocksOnStartup;
        bool m_ShowWelcomeScreen;
        bool m_ShowTOD;
        bool m_CheckDirty;
        quint32 m_UndoStackSize;
        quint32 m_MaxLogEntries;
    } behaviour;

    struct CoverImages {
        bool m_SquareImagesOnly;
        int m_MaximumItemsToQuery;
        bool m_SaveFileNameInDescription;
        bool m_KeepJPGAndPNG;
        int m_ShrinkX;
        int m_ShrinkY;
        bool m_SaveAsJPG;
        bool m_ShrinkImage;
    } coverimages;

    struct Extensions {
        int m_CacheSize;
    } extensions;

    QMap<QString,QString> m_GlobalVars;

private:
};

}

#endif // PREFERENCES_H
