/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include <QAction>

#include <kcompletionbox.h>

#include "completionlineeditor.h"

namespace Texel
{

CompletionLineEditor::CompletionLineEditor(QWidget *parent, int maxCount) :
    KLineEdit(parent),
    m_Completion(new KCompletion()),
    m_MaxCount(maxCount)
{
    setCompletionMode(KGlobalSettings::CompletionPopup);
    setCompletionObject(m_Completion);

    m_Completion->setIgnoreCase(true);
    m_Completion->setOrder(KCompletion::Insertion);

    QAction* allCompletions = new QAction(this);
    allCompletions->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Space));
    allCompletions->setShortcutContext(Qt::WidgetWithChildrenShortcut);
    connect (allCompletions, SIGNAL(triggered()), this, SLOT(showAllCompletions()));
    addAction(allCompletions);
}

void CompletionLineEditor::addCompletionString (const QString& string)
{
    m_InsertOrder << string;
    m_Completion->addItem (string);
    if (m_InsertOrder.size()>m_MaxCount) {
        m_Completion->removeItem(m_InsertOrder.takeFirst());
    }
}

void CompletionLineEditor::showAllCompletions ()
{
    KCompletionBox* box=completionBox();
    box->setItems(m_InsertOrder);
    box->popup();
}

}
