/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "errredirector.h"

#include "model/texeltagmodel.h"
#include "model/tagitem.h"
#include "view/messageview.h"
#include "util/loghook.h"

#include <istream>
#include <sstream>

ErrRedirector::ErrRedirector()
: item(0l)
{
    // Redirect the cerr output stream to an internal stringstream to grab the
    // messages from the taglib. Unfortenately these values are simply dumped
    // in cerr (not thread safe!!!)
    cerr_buffer = std::cerr.rdbuf();
    std::cerr.rdbuf(buf.rdbuf());
}

void ErrRedirector::setItem (Texel::TagItem* _item)
{
    item=_item;
}

ErrRedirector::~ErrRedirector()
{
    std::cerr.rdbuf(cerr_buffer);

    // Process catched error log
    while (!buf.eof()) {
        char b[512];
        buf.getline(b,512);
        if (*b!=0) {
            Texel::LogHook::instance()->warning(QString(b),item);
        }
    }
}

