/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "filedialog.h"
#include "options/preferences.h"

#include <QDir>

#include <kfiledialog.h>


QString Texel::FileDialog::getLoadFileName (const QString& pattern, const QString& title)
{
    const bool useLastDir = Preferences::instance()->filelocs.m_UseLastDirectory;
    QString startDir;

    if (useLastDir) startDir=Preferences::instance()->filelocs.m_LastAccessedDirectory;
    else startDir= Preferences::instance()->filelocs.m_DefaultDirecory;

    const QString filename = KFileDialog::getOpenFileName(KUrl(startDir),pattern,0l,title);

    if (useLastDir) {
        QDir dir(filename);
        Preferences::instance()->filelocs.m_LastAccessedDirectory=dir.absolutePath();
    }

    return filename;
}

QString Texel::FileDialog::getSaveFileName (const QString& pattern, const QString& title)
{
    const bool useLastDir = Preferences::instance()->filelocs.m_UseLastDirectory;
    QString startDir;

    if (useLastDir) startDir=Preferences::instance()->filelocs.m_LastAccessedDirectory;
    else startDir= Preferences::instance()->filelocs.m_DefaultDirecory;

    const QString filename = KFileDialog::getSaveFileName(KUrl(startDir),pattern,0l,title);

    if (useLastDir) {
        QDir dir(filename);
        Preferences::instance()->filelocs.m_LastAccessedDirectory=dir.absolutePath();
    }

    return filename;
}

QString Texel::FileDialog::getExistingDirectory (const QString& title)
{
    const bool useLastDir = Preferences::instance()->filelocs.m_UseLastDirectory;
    QString startDir;

    if (useLastDir) startDir=Preferences::instance()->filelocs.m_LastAccessedDirectory;
    else startDir= Preferences::instance()->filelocs.m_DefaultDirecory;

    const QString dirName = KFileDialog::getExistingDirectory(KUrl(startDir),0l,title);

    if (useLastDir) {
        Preferences::instance()->filelocs.m_LastAccessedDirectory=dirName;
    }

    return dirName;
}
