/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PERSISTENTCACHE_H
#define PERSISTENTCACHE_H

#include <QString>
#include <QCache>
#include <QPair>
#include <QMutex>

#include <ios>

#include "util/sqlitewrapper.h"
#include "util/singleton.h"

namespace Texel {

class PersistentCache: public Singleton<PersistentCache>
{
public:
    void insert (int type, const QString& key, const QByteArray& content) throw (SqLite::Exception);
    QByteArray lookup (int type, const QString& key) throw (SqLite::Exception);

    void compress () throw (SqLite::Exception);

    void setMaxSize (int bytes);

    ~PersistentCache();

private:
    PersistentCache () throw (SqLite::Exception);

    void createStructure () throw (SqLite::Exception);
    int getCachedSize () throw (SqLite::Exception);

    int m_Inserts;

    SqLite::Connection* m_CacheDB;
    SqLite::Statement*  m_InsertStatement;
    SqLite::Statement*  m_DeleteStatement;
    SqLite::Statement*  m_LookupStatement;
    SqLite::Statement*  m_UpdateLastAccessStatement;
    SqLite::Statement*  m_CalcSize;

    typedef QPair<int, QString> Key;

    QCache<Key, QByteArray> m_MemoryCache;

    QMutex m_Mutex;
    friend class Singleton<PersistentCache>;
};

}


#endif // PERSISTENTCACHE_H
