/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef QTLEVENSHTEIN_H
#define QTLEVENSHTEIN_H

#include <QString>
#include <QList>
#include <QVector>

#include <ostream>

namespace Texel {

int distance(const QString& source, const QString& target, int max=0xffff);

namespace Levenshtein {
    class Node;
}

/** This class allows to check for duplicates agaist a large number of samples.
  */
class LevenstheinDistanceTree
{
    Levenshtein::Node* root;

public:
    struct Item
    {
        const QString title;
        const QString album;
        const QString artist;

        Item (const QString& title_, const QString& album_, const QString& artist_)
        : title(title_), album(album_), artist(artist_)
        {
        }

        QChar operator[] (int i) const
        {
            if (i<title.size()) return title[i];
            if (i==title.size()) return 0;
            i-=(title.size()+1);
            if (i<album.size()) return album[i];
            if (i==album.size()) return 0;
            i-=(album.size()+1);
            return artist[i];
        }

        int size() const
        {
            return artist.size()+album.size()+title.size()+2;
        }

        void out (std::ostream& o) const;
    };

    LevenstheinDistanceTree ();
    ~LevenstheinDistanceTree ();

    static int nodes();

    void insert (int n, const Item& name);
    QList<int> findMinimalDistanceItems (int n, const Item& name, int max, int maxErrorsInARow=0) const;
};

inline std::ostream& operator<< (std::ostream& out, const LevenstheinDistanceTree::Item& item)
{
    item.out(out);
    return out;
}

}

#endif // QTLEVENSHTEIN_H
