/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

// taken from http://de.wikibooks.org/wiki/C++-Programmierung:_Entwurfsmuster:_Singleton

#ifndef SINGLETON_H
#define SINGLETON_H

namespace Texel {

template <typename N>
class Singleton
{
public:
    static N* instance ()
    {
       static CGuard g;
       if (!_instance)
          _instance = new N ();
       return _instance;
    }

    static void destroy ()
    {
        delete Singleton<N>::_instance;
        Singleton<N>::_instance = 0L;
    }

protected:
    Singleton () { }
    ~Singleton () { }

private:
    static N* _instance;
    Singleton (const Singleton &);

    class CGuard
    {
    public:
       ~CGuard()
       {
          if (0L!= Singleton<N>::_instance )
          {
             delete Singleton<N>::_instance;
             Singleton<N>::_instance = 0L;
          }
       }
    };

    friend class CGuard;
};

template <typename C> C* Singleton <C>::_instance = 0;

}

#endif // SINGLETON_H
