#ifndef SLIMINDEX_H
#define SLIMINDEX_H

namespace Texel {

struct SlimIndex
{
    SlimIndex (const QModelIndex& index)
    : m_row(index.row()),
      m_col(index.column())
    {
    }

    SlimIndex (int r, int c)
    : m_row(r), m_col(c)
    {
    }

    SlimIndex ()
    : m_row(-1), m_col(-1)
    {
    }

    int m_row;
    int m_col;

    bool isValid () const
    {
        return m_row>=0 && m_col>=0;
    }

    int row () const { return m_row; }
    int column() const { return m_col; }

    void setCol (int c)
    {
        m_col=c;
    }

    void decCol ()
    {
        --m_col;
    }

    void incCol ()
    {
        ++m_col;
    }

    void decRow ()
    {
        --m_row;
    }

    void incRow ()
    {
        ++m_row;
    }

    QModelIndex qmodelindex(QAbstractItemModel* model) const
    {
        if (!isValid()) return QModelIndex();
        return model->index(m_row,m_col);
    }

    bool operator== (const SlimIndex& s) const
    {
        return m_row==s.m_row && m_col==s.m_col;
    }

    bool operator< (const SlimIndex& s) const
    {
        if (m_row==s.m_row) return m_col<s.m_col;
        return m_row<s.m_row;
    }
};

inline bool operator== (const QModelIndex& a, const SlimIndex& b)
{
    return a.row()==b.row() && a.column()==b.column();
}

}

#endif // SLIMINDEX_H
