/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "slowclicktoolbutton.h"
#include "aid.h"

#include <QMouseEvent>
#include <QAction>

using namespace std;

namespace Texel {

SlowClickToolButton::SlowClickToolButton (QWidget* parent)
    : QToolButton (parent),
      m_SlowClickAction(0l)
{
}

SlowClickToolButton::~SlowClickToolButton ()
{
}

void SlowClickToolButton::mousePressEvent (QMouseEvent* event)
{
    if (event->button()==Qt::LeftButton && hitButton(event->pos())) {
        m_PlayPressStart=QTime::currentTime();
        m_Timer=startTimer(300);
    }
    QToolButton::mousePressEvent (event);
}

void SlowClickToolButton::mouseReleaseEvent (QMouseEvent* event)
{
    killTimer(m_Timer);
    if (event->button()==Qt::LeftButton && hitButton(event->pos())) {
        if (m_PlayPressStart.msecsTo (QTime::currentTime())>300) {
            if (m_Timer!=0) {
                emit slowReleased();
                if (m_SlowClickAction!=0l) {
                    m_SlowClickAction->trigger();
                }
				setDown(false);
			}
        }
        else {
            click();
        }
    }
    else QToolButton::mouseReleaseEvent (event);
}

void SlowClickToolButton::setSlowClickAction(QAction *action)
{
    m_SlowClickAction=action;
}

void SlowClickToolButton::timerEvent (QTimerEvent*)
{
    setDown(false);
    emit slowReleased();
    if (m_SlowClickAction!=0l) {
        m_SlowClickAction->trigger();
    }
    killTimer(m_Timer);
    m_Timer=0;
}

ClickableLabel::ClickableLabel (QWidget* parent)
    :QLabel(parent)
{
}

ClickableLabel::~ClickableLabel ()
{
}

void ClickableLabel::mousePressEvent (QMouseEvent* event)
{
    if (event->button()==Qt::LeftButton && rect().contains(event->pos())) emit pressed();
    QLabel::mouseReleaseEvent (event);
}

}

#include "slowclicktoolbutton.moc"
