#ifndef SQLITEWRAPPER_H
#define SQLITEWRAPPER_H

#include <QString>
#include <QByteArray>

#include <sqlite3.h>

namespace Texel {
namespace SqLite {
    struct Exception
    {
        int errno;
        QString msg;
        QString details;

        static void throw_exception (sqlite3* db, int errnum, const QString& desc=QString::null, const QString &details=QString::null);
    };

    class Statement;

    class Binder
    {
    public:
        Binder(Statement* stmt_);

        Binder& operator() (const QString& val);
        Binder& operator() (int val);
        Binder& operator() (const QByteArray& val);
        void reset();
        void execute (bool reset=true);

    private:
        Statement* stmt;
        int col;
    };

    class Statement
    {
    public:
        ~Statement ();

        void reset () throw (Exception);

        void bind (int index, int value) throw (Exception);
        void bind (int index, const QString& value) throw (Exception);
        void bind (int index, const QByteArray& value) throw (Exception);
        Binder& bind();
        void bind (int index) throw (Exception);
        void executeOnce () throw (Exception);
        Statement* execute () throw (Exception);
        bool hasRow () const;
        int getInt (int col) const throw (Exception);
        QString getString (int col) const throw (Exception);
        QByteArray getBlob (int col) const throw (Exception);
        bool isNull (int col) const;

    private:
        Statement (); // Forbidden
        Statement (sqlite3* connection, const char* sql);

        sqlite3_stmt* stmt;
        sqlite3* db;
        int m_Result;
        Binder binder;
        QString sql;

        friend class Connection;
    };

    struct Connection
    {
        Connection (const QString& filename) throw (Exception);
        ~Connection ();

        Statement* prepare (const char* sql) throw (Exception);

        void beginTransaction () throw (Exception);
        void commit () throw (Exception);
        void rollback () throw (Exception);

        Statement* m_Commit;
        Statement* m_Rollback;
        Statement* m_BeginTransaction;

        sqlite3* db;
    };
}
}

#endif // SQLITEWRAPPER_H
