/**
 *
 * Taken from http://stackoverflow.com/questions/1956542/how-to-make-item-view-render-rich-html-text-in-qt
 *
 **/

#include "htmldelegate.h"

#include <QTextDocument>
#include <QAbstractTextDocumentLayout>
#include <QApplication>
#include <QPainter>

#include <algorithm>

#include "aid.h"

HtmlDelegate::HtmlDelegate(QObject *parent) :
    QStyledItemDelegate(parent),
    m_GridSize(-1,-1),
    m_IconSize(-1,-1)
{
}

void HtmlDelegate::paint (QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyleOptionViewItemV4 optionV4 = option;
    initStyleOption(&optionV4, index);

    QStyle *style = optionV4.widget? optionV4.widget->style() : QApplication::style();

    if (m_IconSize.width()!=-1 && m_IconSize.height()!=-1) {
        optionV4.decorationSize=m_IconSize;
        optionV4.decorationAlignment=Qt::AlignCenter;
    }
    QRect textRect = style->subElementRect(QStyle::SE_ItemViewItemText, &optionV4);

    QTextDocument doc;
    doc.setHtml(optionV4.text);
    doc.setTextWidth(textRect.width());

    /// Painting item without text
    optionV4.text = QString();
    style->drawControl(QStyle::CE_ItemViewItem, &optionV4, painter);

    QAbstractTextDocumentLayout::PaintContext ctx;

    // Highlighting text if item is selected
    if (optionV4.state & QStyle::State_Selected)
        ctx.palette.setColor(QPalette::Text, optionV4.palette.color(QPalette::Active, QPalette::HighlightedText));

    painter->save();
    painter->translate(textRect.topLeft());
    painter->setClipRect(textRect.translated(-textRect.topLeft()));
    doc.documentLayout()->draw(painter, ctx);
    painter->restore();
}

QSize HtmlDelegate::sizeHint (const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (m_GridSize.width()!=-1 && m_GridSize.height()!=-1) {
        return m_GridSize;
    }

    QStyleOptionViewItemV4 optionV4 = option;
    initStyleOption(&optionV4, index);

    QTextDocument doc;
    doc.setHtml(optionV4.text);

    int width=(int)doc.idealWidth();
    int height=(int)doc.size().height();

    int iconWidth=0;
    int iconHeight=0;

    if (optionV4.showDecorationSelected && optionV4.features&QStyleOptionViewItemV3::HasDecoration) {
        iconWidth=m_IconSize.width()==-1?option.decorationSize.width():m_IconSize.width();
        iconHeight=m_IconSize.height()==-1?option.decorationSize.height():m_IconSize.height();
    }
    if (option.decorationPosition==QStyleOptionViewItem::Left || option.decorationPosition==QStyleOptionViewItem::Right) {
        width+=iconWidth;
        height=std::max(height,iconHeight);
    }
    if (option.decorationPosition==QStyleOptionViewItem::Bottom || option.decorationPosition==QStyleOptionViewItem::Top) {
        height+=iconHeight;
        width=std::max(width,iconWidth);
    }

    return QSize(width,height);
}
