/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "keytableeditor.h"

#include <QStandardItemModel>

#include <klocale.h>

#include "aid.h"

using namespace std;

namespace Texel {

    KeyTableEditor::KeyTableEditor(QWidget *parent) :
            TableEditor(parent)
    {
        // Save to call from here as model call is known
        init();
        cout << "Columns: " << model() << endl;
    }

    QAbstractItemModel* KeyTableEditor::doInitModel (QWidget* parent) const
    {
        QStandardItemModel* model=new QStandardItemModel(parent);
        model->setColumnCount(2);
        model->setHorizontalHeaderItem(0,new QStandardItem(i18n("Property")));
        model->setHorizontalHeaderItem(1,new QStandardItem(i18n("Value")));

        return model;
    }

    void KeyTableEditor::setKeyValues(const KeyValueMap& map)
    {
        QStandardItemModel* m=dynamic_cast<QStandardItemModel*>(model());
        if (m!=0l) {
            m->removeRows(0,m->rowCount());
            foreach (QString key, map.keys()) {
                QList<QStandardItem*> row;
                row << new QStandardItem(key) << new QStandardItem(map[key]);
                m->appendRow(row);
            }
        }
    }

    KeyTableEditor::KeyValueMap KeyTableEditor::getKeyValues () const
    {
        KeyValueMap keyValues;
        QStandardItemModel* m=dynamic_cast<QStandardItemModel*>(model());
        if (m!=0l) {
            for (int row =0;row<m->rowCount();row++) {
                keyValues.insert(m->data(m->index(row,0)).toString(),m->data(m->index(row,1)).toString());
            }
        }

        return keyValues;
    }

}

#include "keytableeditor.moc"
