/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "tableeditor.h"
#include "ui_tableeditor.h"

#include <QModelIndex>
#include <QStandardItemModel>
#include <QAbstractButton>

#include "aid.h"

using namespace std;

namespace Texel {

    TableEditor::TableEditor(QWidget *parent) :
            QWidget(parent)
    {
    }

    void TableEditor::init ()
    {
        setupUi(this);
        QAbstractItemModel* m=doInitModel (tableView);
        tableView->setModel(m);

        connect (add_Button,SIGNAL(clicked()), this, SLOT(append()));
        connect (remove_Button,SIGNAL(clicked()), this, SLOT(remove()));
    }

    TableEditor::~TableEditor()
    {
    }

    QAbstractItemModel* TableEditor::doInitModel (QWidget* parent) const
    {
        return new QStandardItemModel(parent);
    }

    void TableEditor::changeEvent(QEvent *e)
    {
        QWidget::changeEvent(e);
        switch (e->type()) {
        case QEvent::LanguageChange:
            retranslateUi(this);
            break;
        default:
            break;
        }
    }

    QAbstractItemModel* TableEditor::model() const
    {
        return tableView->model();
    }

    void TableEditor::append ()
    {
        const int row=doAppend();
        // Set UI focus to new cell and start editing
        const QModelIndex index=model()->index(row,0);
        tableView->setCurrentIndex(index);
        tableView->edit (index);
    }

    void TableEditor::remove ()
    {
        const QModelIndexList selectedRows=tableView->selectionModel()->selectedRows(0);
        foreach (QModelIndex idx, selectedRows) {
            doRemove (idx.row());
        }
    }

    void TableEditor::doRemove (int row)
    {
        tableView->model()->removeRow(row);
    }

    int TableEditor::doAppend ()
    {
        QStandardItemModel* m=dynamic_cast<QStandardItemModel*>(model());
        if (m!=0l) {
            QList<QStandardItem*> row;
            for (int i=0;i<m->columnCount();i++) {
                row << new QStandardItem();
            }
            m->appendRow(row);
            return m->rowCount()-1;
        }
        return -1;
    }

    void TableEditor::addButton (QAbstractButton* button)
    {
        button->setParent(tableView);
        buttonBar->addWidget(button);
    }

    QTableView* TableEditor::table() const
    {
        return tableView;
    }

    QTableView* TableEditor::createTableView (QWidget* parent) const
    {
        return new QTableView(parent);
    }

    void TableEditor::setupUi(QWidget *TableEditor)
    {
        if (TableEditor->objectName().isEmpty())
            TableEditor->setObjectName(QString::fromUtf8("TableEditor"));
        TableEditor->resize(400, 300);
        verticalLayout = new QVBoxLayout(TableEditor);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        tableView = createTableView(TableEditor);
        tableView->setObjectName(QString::fromUtf8("tableView"));
        tableView->setAlternatingRowColors(true);
        tableView->setSortingEnabled(true);
        tableView->setCornerButtonEnabled(false);
        tableView->horizontalHeader()->setStretchLastSection(true);

        verticalLayout->addWidget(tableView);

        buttonBar = new QHBoxLayout();
        buttonBar->setObjectName(QString::fromUtf8("buttonBar"));
        add_Button = new QPushButton(TableEditor);
        add_Button->setObjectName(QString::fromUtf8("addButton"));

        buttonBar->addWidget(add_Button);

        remove_Button = new QPushButton(TableEditor);
        remove_Button->setObjectName(QString::fromUtf8("removeButton"));

        buttonBar->addWidget(remove_Button);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        buttonBar->addItem(horizontalSpacer);
        verticalLayout->addLayout(buttonBar);

        retranslateUi(TableEditor);

        QMetaObject::connectSlotsByName(TableEditor);
    } // setupUi

    void TableEditor::retranslateUi(QWidget *TableEditor)
    {
        TableEditor->setWindowTitle(tr2i18n("Form", 0));
        add_Button->setText(tr2i18n("Add", 0));
        remove_Button->setText(tr2i18n("Remove", 0));
    } // retranslateUi

}

#include "tableeditor.moc"
