/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef DETAILEDATTRIBUTETABLE_H
#define DETAILEDATTRIBUTETABLE_H

#include "util/widgets/tableeditor.h"
#include "model/tagextension.h"

#include <QList>
#include <QString>
#include <QStyledItemDelegate>
#include <QStandardItemModel>

class QStandardItem;

namespace Texel {

    class StandardItemUndoModel: public QStandardItemModel
    {
        Q_OBJECT
    public:
        StandardItemUndoModel (TagExtension* ext, QWidget *parent = 0);
        virtual ~StandardItemUndoModel ();
        virtual bool setData (const QModelIndex& index, const QVariant& value, int role=Qt::EditRole);

        void reset (const QString& name, const QVariant& value);
        void remove (const QString& name);
        QList<QStandardItem*> createRow (const QString& key_, const QString& name);

        void setExtension (TagExtension* extension);

    signals:
        void attributeChanged (const QString& attrib, const QVariant& old, const QVariant& upd);

    private:
        TagExtension* m_Extension;
    };

    class DetailedAttributeTable : public TableEditor
    {
        Q_OBJECT
    public:
        explicit DetailedAttributeTable(QWidget *parent = 0);

        void setExtension (TagExtension* extension);

    signals:

    public slots:
        void appendRow ();
        void onChangedData(const QString& attrib, const QVariant& old, const QVariant& upd);

    private:
        virtual QAbstractItemModel* doInitModel (QWidget* parent) const;
        virtual int doAppend ();
        virtual void doRemove (int row);

        void createAddMenu ();

        TagExtension* m_Extension;
    };

    namespace Internal
    {
        /// This delegate sets additionally an validator to qlineedit editors.
        /// It tries to take validator configuration from the model by using
        /// QTUserRole+1,+2: They shall contain the data type and the length
        class AttributeDelegate: public QStyledItemDelegate
        {
            Q_OBJECT
        public:
            AttributeDelegate (QObject* parent);
            virtual ~AttributeDelegate ();

            virtual QWidget * createEditor ( QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
        };
    }

    inline void StandardItemUndoModel::setExtension (TagExtension* extension)
    {
        m_Extension=extension;
    }

}

#endif // DETAILEDATTRIBUTETABLE_H
