/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "directorydock.h"
#include "ui_directorydock.h"

#include "dirmodel.h"
#include "model/dirtree.h"
#include "model/tagmodelproxy.h"
#include "aid.h"

#include <QDir>

#include <kurlnavigator.h>
#include <kfileplacesmodel.h>
#include <kdeversion.h>

using namespace std;

namespace Texel {

    DirectoryDock::DirectoryDock(QWidget *parent) :
            BaseDock(parent),
            ui(new Ui::DirectoryDock),
            m_ProxyModel(0l)
    {
        ui->setupUi(this);

        DirModel* dirModel = new DirModel(this);
        ui->dirTree->setModel (dirModel);

        QBoxLayout* layout=dynamic_cast<QBoxLayout*>(ui->dockWidgetContents->layout());
        if (layout!=0l) {
            KFilePlacesModel* places = new KFilePlacesModel(this);
            places->removePlace(places->index(4,0));
            m_Navi = new KUrlNavigator(places, KUrl(QDir::separator()), ui->dockWidgetContents);
            layout->insertWidget(0,m_Navi);
            connect (m_Navi, SIGNAL(urlChanged(KUrl)), this, SLOT(onRootUrlChange(KUrl)));
#if KDE_IS_VERSION(4,5,0)
            m_Navi->setLocationUrl(KUrl(QDir::homePath()));
#else
            m_Navi->setUrl(KUrl(QDir::homePath()));
#endif
        }

        connect (ui->dirTree,SIGNAL(clicked(QModelIndex)), this, SLOT(onTreeClick(QModelIndex)));
        connect (ui->dirTree,SIGNAL(doubleClicked(QModelIndex)), this, SLOT(onTreeDoubleClick(QModelIndex)));
    }

    void DirectoryDock::onRootUrlChange (const KUrl& url)
    {
        static_cast<DirModel*>(ui->dirTree->model())->open (url);
        const QModelIndex idx=static_cast<DirModel*>(ui->dirTree->model())->indexForUrl(url);
        ui->dirTree->setRootIndex(idx);
    }

    DirectoryDock::~DirectoryDock()
    {
        delete ui;
    }

    void DirectoryDock::changeEvent(QEvent *e)
    {
        QDockWidget::changeEvent(e);
        switch (e->type()) {
        case QEvent::LanguageChange:
            ui->retranslateUi(this);
            break;
        default:
            break;
        }
    }

    void DirectoryDock::setDirTree(DirTree *dirTree)
    {
        dynamic_cast<DirModel*>(ui->dirTree->model())->setDirTree(dirTree);
        connect (dirTree, SIGNAL(hasTracksInList(QString,bool)), dynamic_cast<DirModel*>(ui->dirTree->model()), SLOT(onPathStateChange(QString)));
    }

    void DirectoryDock::onTreeClick (const QModelIndex& idx)
    {
        if (m_ProxyModel!=0l) {
            DirModel* model = getTreeModel();
            if (model->isLoaded(idx)) {
                const QString& path = model->itemForIndex(idx).localPath();
                TagItem::List items = model->getDirTree()->getTagItems(path,true);
                m_ProxyModel->applyExternalFilter(items);
            }
        }
    }

    void DirectoryDock::onTreeDoubleClick (const QModelIndex& idx)
    {
        if (m_ProxyModel!=0l) {
            DirModel* model = getTreeModel();
            const QString& path = model->itemForIndex(idx).localPath();
            if (model->isLoaded(idx)) {
                TagItem::List items = model->getDirTree()->getTagItems(path,false);
                m_ProxyModel->applyExternalFilter(items);
            }
            else {
                emit loadDir (KUrl(path));
            }
        }
    }


    DirModel* DirectoryDock::getTreeModel () const
    {
        return dynamic_cast<DirModel*>(ui->dirTree->model());
    }

    void DirectoryDock::setProxyModel(TagModelProxy* model)
    {
        m_ProxyModel=model;
    }

}
