/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "dirmodel.h"
#include "model/dirtree.h"
#include "aid.h"

#include <QFont>

#include <kdirlister.h>

using namespace std;

namespace Texel {

    DirModel::DirModel(QObject *parent) :
            KDirModel(parent),
            m_Tree(0l)
    {
        m_DirLister=new KDirLister(this);
        m_DirLister->setDirOnlyMode(true);
        setDirLister(m_DirLister);
    }

    int DirModel::columnCount (const QModelIndex&) const
    {
        return 1;
    }

    QVariant DirModel::data (const QModelIndex& index, int role) const
    {
        if (role==Qt::FontRole) {
            QFont font = KDirModel::data(index,role).value<QFont>();
            if (isLoaded (index)) {
                font.setBold(true);
            }
            return QVariant(font);
        }
        else {
            return KDirModel::data(index,role);
        }
    }

    bool DirModel::isLoaded (const QModelIndex& idx) const
    {
        return m_Tree!=0l && m_Tree->contains(itemForIndex(idx).localPath());
    }

    void DirModel::open (const KUrl &url)
    {
        m_DirLister->openUrl(url);
    }

    void DirModel::onPathStateChange(const QString& path)
    {
        const QModelIndex idx = indexForUrl(KUrl(path));
        if (idx.isValid()) {
            emit dataChanged(idx,idx);
        }
    }

}
