/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "playerdock.h"
#include "ui_playerdock.h"

#include "model/tagitem.h"

#include "func/player/player.h"

namespace Texel {

PlayerDock::PlayerDock(QWidget *parent) :
    BaseDock(parent),
    ui(new Ui::PlayerDock)
{
    ui->setupUi(this);
    Player::Player::instance()->initCompleteToolBar(ui->toolBar);
    ui->seekSlider->setMediaObject(Player::Player::instance()->mediaObject());
    ui->playlist->setTearOff(false);

    connect (ui->playlist, SIGNAL(close()), this, SLOT(close()));

    connect (Player::Player::instance(), SIGNAL(playing(TagItem*)), this, SLOT(showCurrentTitle(TagItem*)));
    connect (Player::Player::instance(), SIGNAL(timeElapsed(QString)), ui->timeLabel, SLOT(setText(QString)));
    showCurrentTitle(0l);
}

void PlayerDock::showCurrentTitle (TagItem* item)
{
    if (item==0l) {
        ui->coverLabel->clear();
        ui->coverLabel->hide();
        ui->titleLabel->setText("<center>"+i18n("Nothing played")+"</center>");
    }
    else {
        ui->titleLabel->setText("<html><h2>"+item->data(ColumnManager::Title)+"</h2>"
                                "<i>"+item->data(ColumnManager::Artist)+"<br/>"
                                     +item->data(ColumnManager::Album)+"</i>"
                                "</html>");

        if (item->supportsCoverImages()) {
            QPixmap px = QPixmap::fromImage(item->getDefaultCoverImage());
            if (!px.isNull()) {
                ui->coverLabel->setPixmap(px.scaled(64,64,Qt::KeepAspectRatio));
                ui->coverLabel->show();
            }
            else ui->coverLabel->hide();
        }
        else {
            ui->coverLabel->hide();
        }

    }
}


}

#include "playerdock.moc"
