/*************************************************************************
*************************************************************************/

#ifndef TRACKDETAILS_H
#define TRACKDETAILS_H

#include "basedock.h"

#include <QModelIndex>
#include <QStyledItemDelegate>

#include "model/tagextension.h"

namespace Ui {
    class TrackDetails;
}

class QPoint;
class QStandardItem;
class QToolButton;

namespace Texel {

    class TagItem;

    class TrackDetails : public BaseDock {
        Q_OBJECT
    public:
        TrackDetails(QWidget *parent = 0);
        ~TrackDetails();

    public slots:
        void entered (const QModelIndex& index);
        void changed (const QModelIndex& from , const QModelIndex& to);

        void onPageChanged (int index);

        void prevLyric();
        void nextLyric();

        void setImageType (int type);
        void nextImage ();
        void prevImage ();
        void nextAvailableImage ();
        void prevAvailableImage ();
        void removeImage ();
        void addImage ();
        void imageDescriptionChanged ();
        void searchForCover ();
        void onCoverSelected (const QString& artist, const QString& album, const QImage& cover);

        void cutImage ();
        void copyImage ();
        void pasteImage ();

        void rotatePane ();

    protected slots:
        void droppedUrlList (const QList<QUrl>& urls);
        void droppedImage (const QImage& image);
        void onAlteredExtension (int row, int type, const QString& name, const QVariant& value);

    private slots:
        void imagePopupMenu (const QPoint& pos);
        void initCurrent (int index);
        void alteredRating (int rating);

    signals:
        void displaysImage (bool hasImage);

    protected:
        virtual void changeEvent(QEvent *e);
        virtual void timerEvent (QTimerEvent* event);

    private:
        void setColumnMap ();
        void update (int col, QWidget* widget);
        void updateExtension ();

        void initImages ();
        void showImage ();
        int  checkMoveButtons();

        void initLyrics();
        void showLyrics();
        void initOverview();
        void initExtra ();

        Ui::TrackDetails *ui;
        TagItem* m_Row;
        TagExtension* m_Extension;
        QMap<int,QWidget*> m_ColumnMapper;

        QStringList m_ImageTypes;
        QStringList::iterator m_CurrentImage;
        int m_ImageCount;

        QList<TagExtension::Lyrics> m_Lyrics;
        QList<TagExtension::Lyrics>::iterator m_CurrentLyrics;

        bool m_StandardInitialized;
        bool m_ArtWorkInitialized;
        bool m_ExtendedAttribsInitialized;

        int m_TimerId;
    };

}

#endif // TRACKDETAILS_H
