/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef texel_messageview_H
#define texel_messageview_H

#include <options/enums.h>

#include <QStatusBar>
#include <QAbstractListModel>
#include <QAbstractProxyModel>
#include <QStringList>

#include <kdialog.h>

class QPushButton;
class QWidget;

namespace Ui {
    class MessageViewer;
}

namespace Texel {
    class TagItem;

    class MessageModel:public QAbstractListModel
    {
        Q_OBJECT

    public:
        static QPixmap getIcon(int level);

        struct Item
        {
            typedef QList<Item*> List;

            Item (int _level, const QString& _message, Texel::TagItem* _tag);
            ~Item ();

            int level;
            QString message;
            Texel::TagItem* tag;
        };

        MessageModel (QWidget* parent=0l);
        virtual ~MessageModel();

        virtual int rowCount (const QModelIndex & parent = QModelIndex()) const;
        virtual int columnCount (const QModelIndex & parent = QModelIndex()) const;
        virtual QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
        QVariant data (Item* item, int col, int role) const;

        Item::List getMessagesForItem (Texel::TagItem* item) const;

    public slots:
        void error (const QString& message);
        void message (Enums::MessageTypes level, const QString& message, TagItem* item);
        void clear ();
        void setCurrentMaxLevel (int current);

        void checkConfig ();

    signals:
        void newMaxFaultLevel(int level);

    private:
        class Private;
        Private* p;

        Enums::MessageTypes m_DisplayedLevel;
        Enums::MessageTypes m_CurrentMaxLevel;
    };
}

#endif // texel_messageview_H
