#include "popupmenucontroller.h"

#include "model/tagitem.h"
#include "model/tagextension.h"
#include "model/texeltagmodel.h"

#include "view/texeltablewidget.h"

#include "func/metatagservices/freedb/cdentry.h"

#include <QAction>

namespace Texel {

PopupMenuController::PopupMenuController(QObject* parent)
    : QObject(parent)
{
}

void PopupMenuController::addLocalImageAsCover()
{
    QAction* _sender = dynamic_cast<QAction*>(sender());
    const QString imgName=_sender->data().toString();
    QImage img;
    if (img.load(imgName)) {
        foreach (TagItem* item, m_SelectedRows) {
            if (item->supportsCoverImages()) {
                TagExtension* ext=item->getExtensions();
                ext->setImage(item->getDefaultImageName(),img);
            }
        }
    }
}

void PopupMenuController::setModifiedContentForAll ()
{
    setModifiedContent (true);
}

void PopupMenuController::setModifiedContentForOne ()
{
    setModifiedContent (false);
}

void PopupMenuController::setModifiedContent (bool forAll)
{
    const QVariant vData=dynamic_cast<QAction*>(sender())->data();
    if (!forAll) {
        m_Widget->model()->setData(m_Index,vData);
    }
    else {
        // Take oldname by reading current data element by standard mechanisms
        const QString& oldName = m_Widget->model()->data(m_Index).toString();
        const QString& newName = vData.toString();
        static_cast<TagModel*>(static_cast<TagModelProxy*>(m_Widget->model())->sourceModel())->changeArtistName(oldName,newName);
    }
}

void PopupMenuController::alterDeferedDeletionMark(const QVariant& state)
{
    TagModelProxy* proxymodel = static_cast<TagModelProxy*>(m_Widget->model());
    QModelIndexList indexes;
    foreach (TagItem* item, m_SelectedRows) {
        QVector<int> r=proxymodel->getRow(item);
        if (!r.isEmpty()) {
            indexes << proxymodel->index(r.first(),ColumnManager::MarkForDeletion);
        }
    }
    proxymodel->setDataBlock(state,indexes);
}

void PopupMenuController::alterDeferedDeletionMark ()
{
    QAction*a=dynamic_cast<QAction*>(sender());
    if (a!=0l) {
        const QVariant state = a->data();
        alterDeferedDeletionMark (state);
    }
}

void PopupMenuController::parseCDDBFile ()
{
    QAction*a=dynamic_cast<QAction*>(sender());
    if (a!=0l) {
        QFile cddbFile(a->data().toString());
        cddbFile.open(QIODevice::ReadOnly);
        CDEntry* cddb=CDEntry::parse(&cddbFile);
        cddb->mapContent(m_SelectedRows,false);
        cddbFile.close();
        delete cddb;
    }
}

}

#include "popupmenucontroller.moc"
