/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef PROBLEMAWAREDELEGATE_H
#define PROBLEMAWAREDELEGATE_H

#include <QItemDelegate>
#include <QTextLayout>

namespace Texel {

/**
  This delegate introduces a new role which is requested by the data metzhod of the model. The new role
  is called Texel::Enums::RoleProblem and allows four different results:
  Texel::Enums::ProblemUnknown doesn't affect the displayed result. It simply indicates that currently
  no prblem is known of. In case of Texel::Enums::ProblemNone the data is checked (which might have
  happened asynchonously). This leads to a small icon shown in the bottim right corner of the cell.
  Texel::Enums::ProblemError and Texel::Enums::ProblemWarning will result in a curlied underlined
  text, in case of a warning in amber, and for errors in red.
  */
class ProblemAwareDelegate: public QItemDelegate
{
Q_OBJECT;

public:
   ProblemAwareDelegate (QWidget* parent);
   virtual ~ProblemAwareDelegate ();
   virtual void paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;

   virtual QSize sizeHint (const QStyleOptionViewItem& option, const QModelIndex& index) const;

protected:
   void myDrawDisplay (QPainter* painter, const QStyleOptionViewItem& option, const QRect& rect, const QModelIndex& index) const;

private:
   mutable QTextCharFormat format;
   mutable QTextOption textOpt;
   mutable long time;
   mutable int count;
};

}

#endif // PROBLEMAWAREDELEGATE_H
