#include "decisiondialog.h"
#include "ui_decisiondialog.h"

#include "directoryremover.h"
#include "tagcomparer.h"

namespace Texel {

DescisionDialog::DescisionDialog(TagModel* model, QWidget *parent) :
    QDialog(parent),
    m_KeepClass(true),
    ui(new Ui::DescisionDialog)
{
    ui->setupUi(this);
    m_Remover=new DirectoryRemover(this);
    m_Comparer=new TagComparer(model,this);

    ui->tabWidget->insertTab (0, m_Comparer, i18n("Rename conflicts"));
    ui->tabWidget->insertTab (1, m_Remover, i18n("Directories to remove"));

    connect (ui->okButton, SIGNAL(clicked()), this, SLOT(close()));
    connect (ui->cancelButton, SIGNAL(clicked()), this, SLOT(close()));
    connect (ui->okButton, SIGNAL(clicked()), m_Comparer, SLOT(onAccept()));
    connect (ui->cancelButton, SIGNAL(clicked()), m_Comparer, SLOT(onReject()));
    connect (ui->okButton, SIGNAL(clicked()), m_Remover, SLOT(removeAll()));
}

DescisionDialog::~DescisionDialog()
{
    delete ui;
}

void DescisionDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DescisionDialog::endDirectorySearchJob ()
{
    m_KeepClass=false;
    if (isHidden()) {
        deleteLater();
    }
}

void DescisionDialog::emptyDirectoryToDelete (const QString& directory)
{
    if (isHidden()) {
        show();
    }
    if (ui->tabWidget->currentIndex()==0 && m_Comparer->isEmpty()) {
        ui->tabWidget->setCurrentIndex(1);
    }
    m_Remover->emptyDirectoryToDelete(directory);
}

void DescisionDialog::addCollision(TagItem *first, const QString& filename)
{
    if (isHidden()) {
        show();
    }
    m_Comparer->add(first,filename);
}

}
