#include "directoryremover.h"
#include "ui_directoryremover.h"

#include <QStandardItemModel>
#include <QDir>

namespace Texel {

DirectoryRemover::DirectoryRemover(QWidget* parent) :
    QDialog(parent),
    ui(new Ui::DirectoryRemover)
{
    ui->setupUi(this);
    m_Directories = new QStandardItemModel(this);
    ui->directoryList->setModel(m_Directories);

    connect (ui->buttonRemoveAll, SIGNAL(clicked()), this, SLOT(removeAll()));
    connect (ui->buttonRemovedMarked, SIGNAL(clicked()), this, SLOT(removeMarked()));
    connect (ui->buttonToggleMark, SIGNAL(clicked()), this, SLOT(toggleMark()));
}

DirectoryRemover::~DirectoryRemover()
{
    delete ui;
}

void DirectoryRemover::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DirectoryRemover::emptyDirectoryToDelete(const QString &directory)
{
    QStandardItem* dir = new QStandardItem(directory);
    dir->setCheckable(true);
    dir->setCheckState(Qt::Unchecked);
    m_Directories->appendRow(dir);

    ui->directoryList->scrollTo(m_Directories->index(m_Directories->rowCount()-1,0));
}

void DirectoryRemover::toggleMark()
{
    for (int i=0;i<m_Directories->rowCount();i++) {
        QStandardItem* item = m_Directories->item(i);
        item->setCheckState(item->checkState()==Qt::Checked?Qt::Unchecked:Qt::Checked);
    }
}

void DirectoryRemover::removeAll()
{
    for (int i=0;i<m_Directories->rowCount();i++) {
        QStandardItem* item = m_Directories->item(i);
        removeDirectory(item->text());
    }
    m_Directories->removeRows(0,m_Directories->rowCount());
}

void DirectoryRemover::removeMarked()
{
    for (int i=m_Directories->rowCount()-1;i>=0;i--) {
        QStandardItem* item = m_Directories->item(i);
        if (item->checkState()==Qt::Checked) {
            removeDirectory(item->text());
            m_Directories->removeRow(i);
        }
    }
}

void DirectoryRemover::removeDirectory (const QString& dir)
{
    QDir d;
    d.rmpath(dir);
}

}

#include "directoryremover.moc"
