/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#ifndef TAGCOMPARER_H
#define TAGCOMPARER_H

#include <QWidget>
#include <QList>
#include <QPair>
#include <QString>
#include <QModelIndex>

#include <model/tagitem.h>
#include <view/tagcomparer/tagfunctor.h>

class QStandardItem;

namespace Ui {
    class TagComparer;
}

namespace Texel {

    class TagModel;

    class TagComparer : public QWidget
    {
        Q_OBJECT

    public:
        explicit TagComparer(TagModel* model, QWidget *parent = 0);
        ~TagComparer();

        void setExecutor (TagFunctor* executor);

        bool isEmpty () const;

    public slots:
        void add (TagItem* first, const QString& filename);

        void unsetAll ();
        void keepOriginals ();
        void overwriteOriginals ();
        void keepNewerFiles ();
        void keepOlderFiles ();

        void filterRows (bool hide);

    protected slots:
        void onComparePairClick (const QModelIndex& index);
        void onAccept ();
        void onReject ();

        void onChangedCheckState (const QModelIndex& from, const QModelIndex& to);
        void filterByText ();

    protected:
        void changeEvent(QEvent *e);

    private:
        void setCheckStateToAll (Qt::CheckState);
        bool isOlderFile (QStandardItem* item1, QStandardItem* item2);
        void showAll ();

        Ui::TagComparer *ui;
        TagModel* m_Model;
        TagModel* m_LocalModel;
        TagFunctor* m_Executor;
        bool m_FilterChecked;
    };

    inline void TagComparer::setExecutor (TagFunctor* executor)
    {
        m_Executor=executor;
    }

}

#endif // TAGCOMPARER_H
