/*************************************************************************

    Texel - tagging like excel ; MP3 Tagger
    Copyright (C) 2009-2013 Jens Ketterer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************/

#include "tagdisplay.h"
#include "ui_tagdisplay.h"

#include "model/tagitem.h"
#include "model/columnmanager.h"

#include <QStandardItemModel>

#include "aid.h"

using namespace std;

namespace Texel {

    TagDisplay::TagDisplay(QWidget *parent) :
            QWidget(parent),
            ui(new Ui::TagDisplay)
    {
        ui->setupUi(this);

        QStandardItemModel* model = new QStandardItemModel(ui->itemTable);

        for (int i=0;i<ColumnManager::instance()->getColumnNumber();i++) {
            QList<QStandardItem*> row;
            row << new QStandardItem(ColumnManager::instance()->getName(i))
                << new QStandardItem()
                << new QStandardItem();
            model->appendRow(row);
        }
        model->setColumnCount(3);
        QStringList headers;
        headers << "Tag" << "New file" << "Original";
        model->setHorizontalHeaderLabels(headers);
        ui->itemTable->setModel(model);

        ui->itemTable->horizontalHeader()->setResizeMode(1,QHeaderView::Stretch);
        ui->itemTable->horizontalHeader()->setResizeMode(2,QHeaderView::Stretch);
        ui->itemTable->horizontalHeader()->setResizeMode(0,QHeaderView::ResizeToContents);

        connect (ui->differentOnlyButton, SIGNAL(toggled(bool)), this, SLOT(onDiffButtonToggled(bool)));
    }

    TagDisplay::~TagDisplay()
    {
        delete ui;
    }

    void TagDisplay::changeEvent(QEvent *e)
    {
        QWidget::changeEvent(e);
        switch (e->type()) {
        case QEvent::LanguageChange:
            ui->retranslateUi(this);
            break;
        default:
            break;
        }
    }

    void TagDisplay::setTagItem (int col, TagItem* item_)
    {
        QFont normal = ui->itemTable->font();
        QFont emphasized = normal;
        emphasized.setBold(true);

        QStandardItemModel* model = dynamic_cast<QStandardItemModel*>(ui->itemTable->model());
        for (int i=0;i<ColumnManager::instance()->getColumnNumber();i++) {
            model->item(i,col)->setText(item_->data(i));
            QFont& choice = areRowAttribsEqual(i)?normal:emphasized;
            for (int j=1;j<model->columnCount();j++) {
                model->item(i,j)->setFont(choice);
            }
        }
    }

    bool TagDisplay::areRowAttribsEqual (int row) const
    {
        QStandardItemModel* model = dynamic_cast<QStandardItemModel*>(ui->itemTable->model());
        QString left = model->item(row,1)->text();
        for (int i=2;i<model->columnCount();i++) {
            if (left!=model->item(row,i)->text()) {
                return false;
            }
        }
        return true;
    }

    void TagDisplay::onDiffButtonToggled (bool diffOnly)
    {
        QStandardItemModel* model = dynamic_cast<QStandardItemModel*>(ui->itemTable->model());
        if (diffOnly) {
            for (int i=0;i<model->rowCount();i++) {
                if (areRowAttribsEqual(i)) {
                    ui->itemTable->hideRow(i);
                }
            }
        }
        else {
            for (int i=0;i<model->rowCount();i++) {
                ui->itemTable->showRow(i);
            }
        }
    }

}

#include "tagdisplay.moc"
