/***********************************************************************
*************************************************************************/

#ifndef texel_TexelTableWidget_H
#define texel_TexelTableWidget_H

#include <QTableView>
#include <QStyledItemDelegate>
#include <QLineEdit>
#include <QRect>

#include "model/tagmodelproxy.h"
#include "view/tableextension/problemawaredelegate.h"

#include <auto_ptr.h>

class QMenu;

namespace Texel
{
    class texel;

    class Del: public ProblemAwareDelegate
    {
        Q_OBJECT

    public:
        Del (QWidget* parent);
        virtual ~Del ();
        virtual void setEditorData (QWidget * editor, const QModelIndex& index) const;
        virtual QWidget* createEditor ( QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index ) const ;

    private slots:
        void nextCompletion ();
        void prevCompletion ();
        void setAround();
        void checkForFormular (const QString& text);

    private:
        void iterCompletion (int inc);

        mutable QLineEdit* lineEditor;
        mutable QModelIndex m_Index;
    };

    class TableWidget : public QTableView
    {
        Q_OBJECT;
        typedef QTableView super;

    public:
        TableWidget (QWidget* w);
        virtual ~TableWidget () {}

        virtual void setModel ( QAbstractItemModel * model );

        void setControler (texel* controler)
        {
            m_Controler=controler;
        }

        void setEditBuddy (QLineEdit* buddy)
        {
            m_EditBuddy=buddy;
        }

        QLineEdit* getEditBuddy () const
        {
            return m_EditBuddy;
        }

        TagModelProxy::iterator begin ()
        {
            return dynamic_cast<TagModelProxy*>(model())->begin();
        }

        TagModelProxy::iterator beginCurrent ()
        {
            return dynamic_cast<TagModelProxy*>(model())->begin(currentIndex());
        }

        TagModelProxy::iterator end ()
        {
            return dynamic_cast<TagModelProxy*>(model())->end();
        }

        TagItem::List getSelectedRowsSorted (bool completeLinesOnly=false) const;

        QWidget* indexWidget ( const QModelIndex & index ) const;

    signals:
        void currentText (const QString& text);
        void currentIndexChanged (const QModelIndex& current, const QModelIndex& previous);

    public slots:
        void currentEditingFinished ();
        void clearSelectedCells ();
        virtual void reset();

        void selectAllInSameDirectory (int row);

    protected slots:
        virtual void currentChanged (const QModelIndex& current, const QModelIndex& previous);
        virtual void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);

        virtual bool viewportEvent ( QEvent * event );
        virtual void commitData ( QWidget * editor );

        void clearHeaders ();
        void addHeader (const QList<int>& rows);

    private slots:
        void contextMenu (const QPoint& pos);
        void headerContextMenu (const QPoint& pos);
        void triggerHideColumn ();
        void triggerSelectColumn ();
        void triggerResizeColumn ();
        void triggerFitToScreen ();
        void writeSelectionToM3U ();
        void addCoverImagesToSelected ();
        void addCoverImagesWhenNonExists();
        void addCoverImagesToAll();
        void onModelFilterStart ();
        void onFinishedCoverFilterJob1 (KJob* job);

        void selectAllInSameDirectory ();
        void setDeferedDeletionTag ();
        void unsetDeferedDeletionTag ();

        void selectAllInSameDirectoryRec ();

#ifdef USE_MUSICBRAINZ_SUPPORT
        void showPUID();
#endif

    protected:
        virtual void dragEnterEvent (QDragEnterEvent * event);
        virtual void mouseDoubleClickEvent (QMouseEvent* event);
        virtual void keyReleaseEvent (QKeyEvent* event);
        virtual QModelIndex moveCursor (CursorAction cursorAction, Qt::KeyboardModifiers modifiers);

    private:
        void addCoverImages (const TagItem::List& rows);
        void deferedDeletionTag (bool set);
        void selectAllInSameDirectoryRec (DirTree::Dir* dir);

        QLineEdit* m_EditBuddy;
        QMenu* m_popup;
        texel* m_Controler;

        // Necessary to recover the cursor postion (current index) after a major change. It can be saved
        // when onModelFilterStart is signalled before any model change takes place
        struct OldPosition {
            QModelIndex m_Index;
            QModelIndex m_SourceIndex;
        };

        std::auto_ptr<OldPosition> m_OldPosition;
    };

}
#endif // TexelTableWidget
